/**
 * @file app.c
 *
 * @brief @{PD application handler source file.@}
 *
 *******************************************************************************
 *
 * Copyright (2014-2020), Cypress Semiconductor Corporation or a subsidiary of
 * Cypress Semiconductor Corporation. All rights reserved.
 *
 * This software, including source code, documentation and related materials
 * (“Software”), is owned by Cypress Semiconductor Corporation or one of its
 * subsidiaries (“Cypress”) and is protected by and subject to worldwide patent
 * protection (United States and foreign), United States copyright laws and
 * international treaty provisions. Therefore, you may use this Software only
 * as provided in the license agreement accompanying the software package from
 * which you obtained this Software (“EULA”).
 *
 * If no EULA applies, Cypress hereby grants you a personal, nonexclusive,
 * non-transferable license to copy, modify, and compile the Software source
 * code solely for use in connection with Cypress’s integrated circuit
 * products. Any reproduction, modification, translation, compilation, or
 * representation of this Software except as specified above is prohibited
 * without the express written permission of Cypress. Disclaimer: THIS SOFTWARE
 * IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress reserves the
 * right to make changes to the Software without notice. Cypress does not
 * assume any liability arising out of the application or use of the Software
 * or any product or circuit described in the Software. Cypress does not
 * authorize its products for use in any products where a malfunction or
 * failure of the Cypress product may reasonably be expected to result in
 * significant property damage, injury or death (“High Risk Product”). By
 * including Cypress’s product in a High Risk Product, the manufacturer of such
 * system or application assumes all risk of such use and in doing so agrees to
 * indemnify Cypress against all liability.
 */

#include <config.h>
#include <pd.h>
#include <dpm.h>
#include <psource.h>
#include <psink.h>
#include <pdo.h>
#include <swap.h>
#include <vdm.h>
#include <app.h>
#include <vdm_task_mngr.h>
#include <timer.h>
#if CCG_HPI_ENABLE
#include <hpi.h>
#endif /* CCG_HPI_ENABLE */
#include <alt_mode_hw.h>
#include <alt_modes_mngr.h>
#include <hal_ccgx.h>
#include <gpio.h>
#include <system.h>
#include <dpm_intern.h>
#if CCG_LOAD_SHARING_ENABLE
#include <loadsharing.h>
#include <power_throttle.h>
#endif /* CCG_LOAD_SHARING_ENABLE */
#if (CCG_TEMP_BASED_VOLTAGE_THROTTLING || CCG_VIN_BASED_VOLTAGE_THROTTLING)
#include <sensor_check.h>
#include <power_throttle.h>
#endif /* (CCG_TEMP_BASED_VOLTAGE_THROTTLING || CCG_VIN_BASED_VOLTAGE_THROTTLING) */
#if CCG_CABLE_COMP_ENABLE
#include <cable_comp.h>
#endif /* CCG_CABLE_COMP_ENABLE */

#if RIDGE_SLAVE_ENABLE
#include <ridge_slave.h>
#include <intel_ridge.h>
#endif /* RIDGE_SLAVE_ENABLE */
#if DP_UFP_SUPP
#include <dp_sid.h>
#endif /* DP_UFP_SUPP */
#if (CCG_BB_ENABLE != 0)
#include <billboard.h>
#endif /* (CCG_BB_ENABLE != 0) */

#if DP_UFP_SUPP
#include <hpd.h>
#endif /* DP_UFP_SUPP */

#if BATTERY_CHARGING_ENABLE
#include <battery_charging.h>
#endif /* BATTERY_CHARGING_ENABLE */

#if (CCG_TYPE_A_PORT_ENABLE == 1)
#include <type_a.h>
#endif /* (CCG_TYPE_A_PORT_ENABLE == 1) */

#if CCG_UCSI_ENABLE
#include <ucsi.h>
#endif /* CCG_UCSI_ENABLE */

#if ICL_ENABLE
#include <icl.h>
#endif /* ICL_ENABLE */

#if BB_RETIMER_ENABLE
#include <bb_retimer.h>
#endif /* BB_RETIMER_ENABLE */

#if CCG_SYNC_ENABLE
#include <ccg_sync.h>
#endif

#if (CCG_LOAD_SHARING_ENABLE && CCG_REV3_HANDLE_BAD_SINK)
bool gl_bad_device[NO_OF_TYPEC_PORTS] = {false};
#endif /* (CCG_LOAD_SHARING_ENABLE && CCG_REV3_HANDLE_BAD_SINK) */

#if ((BATTERY_CHARGING_ENABLE) || (BB_RETIMER_ENABLE))
uint8_t gl_system_state = NB_SYS_PWR_STATE_S0;
#endif /* ((BATTERY_CHARGING_ENABLE) || (BB_RETIMER_ENABLE)) */

#if APP_PPS_SINK_SUPPORT
uint8_t hpi_user_reg_handler(uint16_t addr, uint8_t size, uint8_t *data);
void app_pps_sink_disable(uint8_t port);
#endif /* APP_PPS_SINK_SUPPORT */

#if CONFIG_TABLE_GET_FUNCTIONS_ENABLE
ovp_settings_t* pd_get_ptr_ovp_tbl(uint8_t port)
{
    /* Update the OVP settings from the configuration table. */
    return ((ovp_settings_t *)((uint8_t *)(get_pd_config ()) +
                get_pd_port_config(port)->ovp_tbl_offset));
}
#endif /* CONFIG_TABLE_GET_FUNCTIONS_ENABLE */

#if (!CONFIG_TABLE_GET_FUNCTIONS_ENABLE)

const uint8_t cfg_subtable_loc_offset[] = {
    0x8A,                       /* 0: Offset of VBus OVP sub-table location. */
    0x8C,                       /* 1: Offset of VBus OCP sub-table location. */
    0x90,                       /* 2: Offset of VBus SCP sub-table location. */
    0x9C,                       /* 3: Offset of VBus RCP sub-table location. */
    0x94,                       /* 4: Offset of OTP sub-table location. */
    0x92,                       /* 5: Offset of VConn OCP sub-table location. */
    0xBC,                       /* 6: Offset of ICL/TGL platform sub-table location. */
    0xBA,                       /* 7: Offset of Custom Host sub-table location. */
    0xB6,                       /* 8: Offset of Thunderbolt Host sub-table location. */
    0xBE,                       /* 9: Offset of Custom alt-mode sub-table location. */
    0xB8                        /* 10: Offset of alt-mode sub-table location. */
};

uint8_t *pd_get_ptr_cfg_sub_tbl(uint8_t port, uint8_t type)
{
    uint8_t *base_p = (uint8_t *)(get_pd_config());
    uint16_t *ptr = (uint16_t *)(base_p + 0x20 + (port * 0x00F0) + cfg_subtable_loc_offset[type]);
    return (base_p + *ptr);
}

#else /* CONFIG_TABLE_GET_FUNCTIONS_ENABLE */

ocp_settings_t* pd_get_ptr_ocp_tbl(uint8_t port)
{
    /* Update the VBus OCP settings from the configuration table */
    return ((ocp_settings_t *)((uint8_t *)(get_pd_config ()) +
                get_pd_port_config(port)->ocp_tbl_offset));
}

rcp_settings_t* pd_get_ptr_rcp_tbl(uint8_t port)
{
    /* Update the VBus RCP settings from the configuration table */
    return ((rcp_settings_t *)((uint8_t *)(get_pd_config ()) +
                get_pd_port_config(port)->rcp_tbl_offset));
}

scp_settings_t* pd_get_ptr_scp_tbl(uint8_t port)
{
    /* Update the VBus SCP settings from the configuration table */
    return ((scp_settings_t *)((uint8_t *)(get_pd_config ()) +
                get_pd_port_config(port)->scp_tbl_offset));
}

vconn_ocp_settings_t* pd_get_ptr_vconn_ocp_tbl(uint8_t port)
{
    /* Update the Vcon OCP settings from the configuration table */
    return ((vconn_ocp_settings_t *)((uint8_t *)(get_pd_config ()) +
                get_pd_port_config(port)->vconn_ocp_tbl_offset));
}

otp_settings_t* pd_get_ptr_otp_tbl(uint8_t port)
{
    /* Update the OTP settings from the configuration table */
    return ((otp_settings_t *)((uint8_t *)(get_pd_config ()) +
                get_pd_port_config(port)->otp_tbl_offset));
}

#endif /* CONFIG_TABLE_GET_FUNCTIONS_ENABLE */

uvp_settings_t* pd_get_ptr_uvp_tbl(uint8_t port)
{
    /* Update the VBus UVP settings from the configuration table */
    return ((uvp_settings_t *)((uint8_t *)(get_pd_config ()) +
                get_pd_port_config(port)->uvp_tbl_offset));
}

pwr_params_t* pd_get_ptr_pwr_tbl(uint8_t port)
{
    /* Update the power parameters from the configuration table */
    return ((pwr_params_t *)((uint8_t *)(get_pd_config ()) +
                get_pd_port_config(port)->pwr_tbl_offset));
}

#if (BATTERY_CHARGING_ENABLE) || (QC_4_0_ENABLE)
chg_cfg_params_t* pd_get_ptr_chg_cfg_tbl(uint8_t port)
{
#if CCG_TYPE_A_PORT_ENABLE
    if (port == TYPE_A_PORT_ID)
    {
        /* Return parameters for TYPE-A port. */
        return pd_get_ptr_type_a_chg_cfg_tbl (0);
    }
#endif /* CCG_TYPE_A_PORT_ENABLE */

    /* Update the legacy charging parameters from the configuration table */
    return ((chg_cfg_params_t *)((uint8_t *)(get_pd_config ()) +
        get_pd_port_config(port)->chg_cfg_tbl_offset));
}
#endif /* (BATTERY_CHARGING_ENABLE) || (QC_4_0_ENABLE) */

#if POWER_BANK
bat_chg_params_t* pd_get_ptr_bat_chg_tbl(uint8_t port)
{
    /* Update the battery charging parameterss from the configuration table */
    return ((bat_chg_params_t *)((uint8_t *)(get_pd_config ()) +
                get_pd_port_config(port)->bat_chg_tbl_offset));
}
#endif /* POWER_BANK */

pwr_params_t* pd_get_ptr_type_a_pwr_tbl(uint8_t port)
{
    /* Update the power parameters of Type-A port from the configuration table */
    return ((pwr_params_t *)((uint8_t *)(get_pd_config ()) +
                get_pd_port_config(port)->type_a_pwr_tbl_offset));
}

#if CCG_TYPE_A_PORT_ENABLE
typeA_chg_cfg_params_t* pd_get_ptr_type_a_chg_cfg_tbl(uint8_t port)
{
    /* Update the legacy charging parameters from the configuration table */
    return ((typeA_chg_cfg_params_t *)((uint8_t *)(get_pd_config ()) +
                get_pd_port_config(port)->type_a_chg_tbl_offset));
}
#endif /* CCG_TYPE_A_PORT_ENABLE */

#if (CCG_BB_ENABLE != 0)
bb_settings_t* pd_get_ptr_bb_tbl(uint8_t port)
{
    /* Update the Billboard settings from the configuration table*/
    return ((bb_settings_t *)((uint8_t *)(get_pd_config ()) +
                get_pd_port_config(port)->bb_tbl_offset));
}
#endif /* (CCG_BB_ENABLE != 0) */

#if (defined(CCG3PA) || defined(CCG3PA2) || defined(PAG1S))
auto_cfg_settings_t* pd_get_ptr_auto_cfg_tbl(uint8_t port)
{
    /* Update the Automotive charger settings from the configuration table*/
    return ((auto_cfg_settings_t *)((uint8_t *)(get_pd_config ()) +
                get_pd_port_config(port)->auto_cfg_tbl_offset));
}
#endif /* (defined(CCG3PA) || defined(CCG3PA2) || defined(PAG1S)) */

#if CONFIG_TABLE_GET_FUNCTIONS_ENABLE

tbthost_cfg_settings_t* pd_get_ptr_tbthost_cfg_tbl(uint8_t port)
{
    /* Retrieve the thunderbolt host config parameters. */
    return ((tbthost_cfg_settings_t *)((uint8_t *)(get_pd_config ()) +
                get_pd_port_config(port)->tbthost_cfg_tbl_offset));
}

custom_host_cfg_settings_t* pd_get_ptr_host_cfg_tbl(uint8_t port)
{
    /* Retrieve the custom host config parameters. */
    return ((custom_host_cfg_settings_t *)((uint8_t *)(get_pd_config ()) +
                get_pd_port_config(port)->custom_host_tbl_offset));
}

icl_tgl_cfg_settings_t* pd_get_ptr_icl_tgl_cfg_tbl(uint8_t port)
{
    /* Retrieve the ICL/TGL config parameters. */
    return ( (icl_tgl_cfg_settings_t*) ((uint8_t *) (get_pd_config ()) +
                get_pd_port_config(port)->icl_tgl_tbl_offset));
}
custom_alt_cfg_settings_t* pd_get_ptr_custom_alt_mode_tbl(uint8_t port)
{
    /* Retrieve the custom alt. mode parameters. */
    return ((custom_alt_cfg_settings_t *) ((uint8_t *) (get_pd_config ()) +
                get_pd_port_config(port)->custom_alt_mode_tbl_offset));
}

alt_cfg_settings_t* pd_get_ptr_alt_mode_tbl(uint8_t port)
{
    /* Retrieve the custom alt. mode parameters. */
    return ((alt_cfg_settings_t *) ((uint8_t *) (get_pd_config ()) +
                 get_pd_port_config(port)->alt_mode_tbl_offset));
}

#endif /* CONFIG_TABLE_GET_FUNCTIONS_ENABLE */

#if (CCG_TYPE_A_PORT_ENABLE)
app_status_t app_status[2];
#else
app_status_t app_status[NO_OF_TYPEC_PORTS];
#endif /* CCG_TYPE_A_PORT_ENABLE */

/* Flag to indicate that activity timer timed out. */
static volatile bool ccg_activity_timer_timeout = false;

#if CCG_REV3_HANDLE_BAD_SINK
/* Bad sink timeout status */
volatile uint8_t gl_bad_sink_timeout_status[NO_OF_TYPEC_PORTS];
/* Bad PD sink connected status */
volatile uint8_t gl_bad_sink_pd_status[NO_OF_TYPEC_PORTS];
volatile bool gl_bad_sink_apdo_sel[NO_OF_TYPEC_PORTS];
#endif /* CCG_REV3_HANDLE_BAD_SINK */

bool app_validate_configtable_offsets()
{
    uint8_t  port;

#if CCG_REG_SEC_CTRL
    uint32_t sil_id = 0;
    sys_get_silicon_id(&sil_id);
#endif /* CCG_REG_SEC_CTRL */

    for (port = TYPEC_PORT_0_IDX ; port < NO_OF_TYPEC_PORTS; port++)
    {
#if (defined(CCG3PA) || defined(CCG3PA2) || defined(PAG1S))
        if ((get_pd_port_config(port)->pwr_tbl_offset == 0) ||
            (pd_get_ptr_pwr_tbl(port)->table_len < sizeof(pwr_params_t))
#if CCG_REG_SEC_CTRL
            /* If the part is PAG1S secondary controlled, also check the silicon ID. */
        || ((sil_id != PAG1S_SSC_20W_SID) && (sil_id != PAG1S_SSC_33W_SID) && (sil_id != PAG1S_SSC_45W_SID))
#endif /* CCG_REG_SEC_CTRL */
)
        {
            return false;
        }

        if(VBUS_CTRL_TYPE_P1 != pd_get_ptr_pwr_tbl(port)->fb_type)
        {
            return false;
        }
#endif /* (defined(CCG3PA) || defined(CCG3PA2) || defined(PAG1S)) */

#if (CCG_TEMP_BASED_VOLTAGE_THROTTLING || CCG_VIN_BASED_VOLTAGE_THROTTLING || CCG_LOAD_SHARING_ENABLE)
        if ((get_pd_port_config(port)->auto_cfg_tbl_offset == 0) ||
            (pd_get_ptr_auto_cfg_tbl(port)->table_len < sizeof(auto_cfg_settings_t)))
        {
            return false;
        }
        /* No retries for Auto Table. */
#endif /* (CCG_TEMP_BASED_VOLTAGE_THROTTLING || CCG_VIN_BASED_VOLTAGE_THROTTLING || CCG_LOAD_SHARING_ENABLE) */

        /* Initialize the fault-handler variables. */
        if (fault_handler_init_vars (port) == false)
        {
            return false;
        }

#if OTP_ENABLE
        if (
                (get_pd_port_config(port)->otp_tbl_offset == 0)
#ifndef CCG6
                ||
                (PD_GET_PTR_OTP_TBL(port)->table_len < sizeof(otp_settings_t))
#endif /* CCG6 */
           )
        {
            return false;
        }

        /* No retries for OTP. */
#endif /* OTP_ENABLE */

#if (BATTERY_CHARGING_ENABLE) || (QC_4_0_ENABLE)
#if CCG_TYPE_A_PORT_ENABLE
        if (port == TYPE_A_PORT_ID)
        {
            if ((get_pd_port_config(0)->type_a_chg_tbl_offset == 0) ||
                (pd_get_ptr_type_a_chg_cfg_tbl(0)->table_len < sizeof(typeA_chg_cfg_params_t)))
            {
                return false;
            }
        }
        else
#endif /* CCG_TYPE_A_PORT_ENABLE */
        {
            if ((get_pd_port_config(port)->chg_cfg_tbl_offset == 0)  ||
                (pd_get_ptr_chg_cfg_tbl(port)->table_len < sizeof(chg_cfg_params_t)))
            {
                return false;
            }
        }
#endif /* (BATTERY_CHARGING_ENABLE) || (QC_4_0_ENABLE) */

#if POWER_BANK
        if ((get_pd_port_config(port)->bat_chg_tbl_offset == 0)  ||
            (pd_get_ptr_bat_chg_tbl(port)->table_len < sizeof(bat_chg_params_t)))
        {
            return false;
        }
#endif /* POWER_BANK */

#if (CCG_TYPE_A_PORT_ENABLE == 1)
        if (get_pd_port_config(port)->type_a_enable)
        {
            if ((get_pd_port_config(port)->type_a_pwr_tbl_offset == 0)  ||
                (pd_get_ptr_type_a_pwr_tbl(port)->table_len < sizeof(pwr_params_t)))
            {
                return false;
            }
        }
#endif /* (CCG_TYPE_A_PORT_ENABLE == 1) */

#if (CCG_BB_ENABLE != 0)
        if ((get_pd_port_config(port)->bb_tbl_offset == 0)  ||
            (pd_get_ptr_bb_tbl(port)->table_len < sizeof(bb_settings_t)))
        {
            return false;
        }
#endif /* (CCG_BB_ENABLE != 0) */
    }

    return true;
}

#if ((defined(CCG3PA)) || (defined(CCG3PA2)) || (defined(PAG1S)))
bool ccg_app_is_idle(void)
{
    /*
     * If activity timer timeout event is not pending, CCG is idle and system can
     * enter low power mode.
     */
    return !ccg_activity_timer_timeout;
}

void ccg_activity_timer_cb(uint8_t instance, timer_id_t id)
{
    (void)instance;
    (void)id;
    /*
     * Activity timer expired. Generate an event so that CCG periodic checks
     * can run.
     */
    ccg_activity_timer_timeout = true;
}

void ccg_app_task(uint8_t port)
{
#if CCG_CABLE_COMP_ENABLE
    ccg_cable_comp_task(port);
#endif /* CCG_CABLE_COMP_ENABLE */

#if (CCG_TEMP_BASED_VOLTAGE_THROTTLING || CCG_VIN_BASED_VOLTAGE_THROTTLING)
    ccg_sensor_debounce_task();
#endif /* (CCG_TEMP_BASED_VOLTAGE_THROTTLING || CCG_VIN_BASED_VOLTAGE_THROTTLING) */

#if CCG_LOAD_SHARING_ENABLE
    ccg_ls_task(port);
#endif /* CCG_LOAD_SHARING_ENABLE */

#if (CCG_TEMP_BASED_VOLTAGE_THROTTLING || CCG_VIN_BASED_VOLTAGE_THROTTLING || CCG_LOAD_SHARING_ENABLE)
    ccg_power_throttle_task(port);
#endif /* (CCG_TEMP_BASED_VOLTAGE_THROTTLING || CCG_VIN_BASED_VOLTAGE_THROTTLING || CCG_LOAD_SHARING_ENABLE) */

    /* Check VBATT, OTP and TYPE-A current consumption if activity timer has timed out. */
    if (ccg_activity_timer_timeout == true)
    {
#if (POWER_BANK == 1)
        pb_bat_monitor ();
#endif /* (POWER_BANK == 1) */

#if (CCG_TYPE_A_PORT_ENABLE == 1)
        if (get_pd_port_config(0)->type_a_enable)
        {
            type_a_detect_disconnect ();
        }
#endif /* (CCG_TYPE_A_PORT_ENABLE == 1) */

#if (OTP_ENABLE == 1)
        app_otp_check_temp (port);
#endif /* OTP_ENABLE */

#if (CCG_TEMP_BASED_VOLTAGE_THROTTLING || CCG_VIN_BASED_VOLTAGE_THROTTLING)
        ccg_sensor_check();
#endif /* (CCG_TEMP_BASED_VOLTAGE_THROTTLING || CCG_VIN_BASED_VOLTAGE_THROTTLING) */

#if (defined(PAG1S) && CCG_REG_SEC_CTRL)
        pd_pasc_poll_task(port);
#endif /* (defined(PAG1S) && CCG_REG_SEC_CTRL) */

#if ((OTP_ENABLE == 1) || (POWER_BANK == 1) || (CCG_TYPE_A_PORT_ENABLE == 1) || \
    (CCG_TEMP_BASED_VOLTAGE_THROTTLING == 1) || (CCG_VIN_BASED_VOLTAGE_THROTTLING == 1) || \
    (CCG_LOAD_SHARING_ENABLE == 1) || (CCG_PASC_VALLEY_FW_ALGO_ENABLE == 1))
        ccg_activity_timer_timeout = false;
        timer_start (0, CCG_ACTIVITY_TIMER_ID, CCG_ACTIVITY_TIMER_PERIOD,
                ccg_activity_timer_cb);
#endif /* (OTP_ENABLE == 1) || (POWER_BANK == 1) || (CCG_TYPE_A_PORT_ ENABLE == 1) */
    }
}

void ccg_app_task_init(void)
{
#if (POWER_BANK == 1)
    pb_task_init ();
#endif /* (POWER_BANK == 1) */

#if OTP_ENABLE
    /* Enable OTP. */
    app_otp_enable (NO_OF_TYPEC_PORTS);
#endif /* OTP_ENABLE */

    /*
     * Start CCG activity timer. This timer is periodically used to monitor
     * battrey voltage (in power bank application), TYPE-A current consumption
     * and OTP.
     */
#if ((OTP_ENABLE == 1) || (POWER_BANK == 1) || (CCG_TYPE_A_PORT_ENABLE == 1) || \
    (CCG_TEMP_BASED_VOLTAGE_THROTTLING == 1) || (CCG_VIN_BASED_VOLTAGE_THROTTLING == 1) ||\
    (CCG_LOAD_SHARING_ENABLE == 1) || (CCG_PASC_VALLEY_FW_ALGO_ENABLE == 1))
    ccg_activity_timer_timeout = false;
    timer_start (0, CCG_ACTIVITY_TIMER_ID, CCG_ACTIVITY_TIMER_PERIOD,
            ccg_activity_timer_cb);
#endif /* (OTP_ENABLE == 1) || (POWER_BANK == 1) || (CCG_TYPE_A_PORT_ENABLE == 1) */
}
#endif /* (defined(CCG3PA) || defined(CCG3PA2) || defined(PAG1S)) */

#if ((DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP))
static bool app_is_vdm_task_ready(uint8_t port)
{
    /* Assume cable discovery finished when device is UFP. */
    bool retval = true;

#if DFP_ALT_MODE_SUPP

    const dpm_status_t *dpm_stat = dpm_get_info (port);

    /* This check only makes sense for DFP. */
    if (gl_dpm_port_type[port] != PRT_TYPE_UFP)
    {
#if (ROLE_PREFERENCE_ENABLE)
        /* Don't proceed with alternate mode if DR_SWAP is pending. */
        if ((app_status[port].app_pending_swaps & APP_DR_SWAP_PENDING) != 0)
        {
            return false;
        }
#endif /* (ROLE_PREFERENCE_ENABLE) */

#if (!CCG_CBL_DISC_DISABLE)
        /*
         * Set the cable discovered flag if:
         * 1. Cable discovery is disabled.
         * 2. EMCA present flag in DPM is set.
         * 3. Cable discovery process not restarted.
         */
        if (
                (dpm_stat->cbl_dsc == false) ||
               ((dpm_stat->emca_present != false) &&
                (app_status[port].disc_cbl_pending == false))
            )
        {
            app_status[port].cbl_disc_id_finished = true;
        }

        /* Return the status of Cable discovered flag. */
        retval = app_status[port].cbl_disc_id_finished;
#endif /* (!CCG_CBL_DISC_DISABLE) */
    }

#endif /* DFP_ALT_MODE_SUPP */

    return retval;
}
#endif /* ((DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP)) */

ccg_status_t app_disable_pd_port(uint8_t port, dpm_typec_cmd_cbk_t cbk)
{
    ccg_status_t retval = CCG_STAT_SUCCESS;

    if (timer_is_running (port, APP_FAULT_RECOVERY_TIMER))
    {
        /* If the HPI Master is asking us to disable the port, make sure all fault protection state is cleared. */
        app_status[port].fault_status &= ~(
                APP_PORT_VBUS_DROP_WAIT_ACTIVE | APP_PORT_SINK_FAULT_ACTIVE | APP_PORT_DISABLE_IN_PROGRESS |
                APP_PORT_VCONN_FAULT_ACTIVE | APP_PORT_V5V_SUPPLY_LOST);
        timer_stop(port, APP_FAULT_RECOVERY_TIMER);
        pd_typec_dis_rd(port, CC_CHANNEL_1);
        pd_typec_dis_rd(port, CC_CHANNEL_2);
        cbk(port, DPM_RESP_SUCCESS);
    }
    else
    {
        /* Just pass the call on-to the stack. */
        if (dpm_get_info(port)->dpm_enabled)
        {
            retval = dpm_typec_command(port, DPM_CMD_PORT_DISABLE, cbk);
        }
        else
        {
            cbk(port, DPM_RESP_SUCCESS);
        }
    }

    return retval;
}

#if CCG_REV3_HANDLE_BAD_SINK
/* Bad sink timer callback */
void app_bad_sink_timeout_cbk(uint8_t port, timer_id_t id)
{
    (void)id;
    const dpm_status_t* dpm_stat = dpm_get_info(port);

    /* Take bad sink action only if PD connected device and PD is REV3. */
    if ((((dpm_stat->pd_connected == true) ||
         (gl_bad_sink_pd_status[port] == true)) &&
         (gl_bad_sink_timeout_status[port] == false)) || (gl_bad_sink_apdo_sel[port] == true))
    {
        /* Set bad sink status and retain it until port detach or disable */
        gl_bad_sink_apdo_sel[port] = false;
        gl_bad_sink_timeout_status[port] = true;
#if (CCG_LOAD_SHARING_ENABLE)
        gl_bad_device[port] = true;
        ccg_power_throttle_set_stop_ls(port, true);
        ccg_power_throttle_set_stop_power_throttle(port, true);
#endif /* (CCG_LOAD_SHARING_ENABLE) */
        /* Initiate Type-C error recovery. */
        dpm_typec_command(port, DPM_CMD_TYPEC_ERR_RECOVERY, NULL);
    }
}
#endif /* CCG_REV3_HANDLE_BAD_SINK */

extern void ccg_hal_check_if_vsys_up (void);

#if (!CCG_CBL_DISC_DISABLE)
static void app_cbl_dsc_timer_cb (uint8_t port, timer_id_t id);
static void app_cbl_dsc_callback (uint8_t port, resp_status_t resp, const pd_packet_t *pkt_ptr)
{
    /* Keep repeating the DPM command until we succeed. */
    if (resp == SEQ_ABORTED)
    {
        timer_start (port, APP_CBL_DISC_TRIGGER_TIMER, APP_CBL_DISC_TIMER_PERIOD, app_cbl_dsc_timer_cb);
    }
}

static void app_cbl_dsc_timer_cb (uint8_t port, timer_id_t id)
{
    if (dpm_pd_command (port, DPM_CMD_INITIATE_CBL_DISCOVERY, NULL, app_cbl_dsc_callback) != CCG_STAT_SUCCESS)
    {
        /* Start timer which will send initiate the DPM command after a delay. */
        app_cbl_dsc_callback(port, SEQ_ABORTED, 0);
    }
}
#endif /* (!CCG_CBL_DISC_DISABLE) */

#if ((DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP))

static void app_exit_all_cbk (uint8_t port, timer_id_t id)
{
    /* Reset VDM layer */
    app_vdm_layer_reset(port);
}

static void app_exit_alt_modes_cbk(uint8_t port, uint32_t stat)
{
    /* Run timer to run VDM layer restart */
    timer_start (port, APP_RESET_VDM_LAYER_TIMER, APP_RESET_VDM_TIMER_PERIOD, app_exit_all_cbk);
}
#endif /* (DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP) */

bool app_vdm_layer_reset(uint8_t port)
{
#if ((DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP))
    const dpm_status_t *dpm_stat = dpm_get_info(port);
    app_status_t *app = app_get_status(port);
    bool stat = true;

    if ((dpm_stat->contract_exist) && (gl_dpm_port_type[port] == PRT_TYPE_DFP))
    {
        if (app->alt_mode_entered == false)
        {
            /*
             * Reset the alternate mode state machine. The cable discovery complete flag is also cleared so
             * that alternate mode state machine can be started at the end of cable discovery.
             */
            alt_mode_layer_reset(port);
        }
        else
        {
            /* Exit all alt modes at first by sending Exit VDM */
            alt_mode_mngr_exit_all(port, true, app_exit_alt_modes_cbk);
            return stat;
        }
#if (!CCG_CBL_DISC_DISABLE)
        app->cbl_disc_id_finished = false;
        app->disc_cbl_pending     = true;

        /* Ask PD stack to trigger cable discovery. */
        if (dpm_pd_command(port, DPM_CMD_INITIATE_CBL_DISCOVERY,
                    NULL, app_cbl_dsc_callback) != CCG_STAT_SUCCESS)
        {
            timer_start(port, APP_CBL_DISC_TRIGGER_TIMER,
                    APP_CBL_DISC_TIMER_PERIOD, app_cbl_dsc_timer_cb);
        }
#endif /* (!CCG_CBL_DISC_DISABLE) */
    }
    else
    {
#if VCONN_OCP_ENABLE
        /* If there is no PD contract in place and we are VConn source, enable VConn and move on. */
        if ((dpm_stat->attach) && (dpm_stat->vconn_logical))
        {
            vconn_enable (port, dpm_stat->rev_pol);
        }
#endif /* VCONN_OCP_ENABLE */

        stat = false;
    }
    return stat;
#else
    return false;
#endif /* (DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP) */
}

uint8_t app_task(uint8_t port)
{
    fault_handler_task (port);

#if CCG_REV3_HANDLE_BAD_SINK
    /*
     * Check for bad sink timeout.
     * If the PD contract was not completed for APP_BAD_SINK_TIMEOUT_TIMER_PERIOD,
     * a Type-C error recovery was initiated. Downgrade the PD revision from
     * REV3 to REV2 as expected by few bad sinks to complete the contract.
     * Bad sink status will be maintained until port disconnect/disable.
     */
    if((gl_bad_sink_timeout_status[port] == true) && (dpm_get_info(port)->pe_fsm_state == PE_FSM_SRC_STARTUP))
    {
        dpm_downgrade_pd_port_rev(port);
    }
#endif /* CCG_REV3_HANDLE_BAD_SINK */

#if ((DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP))
    /* If VDM processing is allowed */
    if (app_status[port].vdm_task_en != false)
    {
        /* Wait for cable discovery completion before going on Alt. Modes. */
        if (app_is_vdm_task_ready (port))
        {
            vdm_task_mngr (port);
        }
    }
#endif /* (DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP) */

#if NCP_CLIND_ENABLE
    if(port == TYPEC_PORT_0_IDX)
    {
        Clind_OCP_Check(port);
    }
#endif /* NCP_CLIND_ENABLE */

#if (CCG_BB_ENABLE != 0)
    if (bb_is_present(port) != false)
    {
        bb_task(port);
    }
#endif /* (CCG_BB_ENABLE != 0) */

#if MUX_DELAY_EN
    if (
            (app_mux_update_req[port] != false) &&
            (app_status[port].is_mux_busy == false)
       )
    {
        set_mux (port, app_mux_saved_state[port], app_mux_saved_custom_data[port]);
    }
#endif /* MUX_DELAY_EN */

#if BATTERY_CHARGING_ENABLE
    uint8_t i;
    for (i = 0; i < NO_OF_BC_PORTS; i++)
    {
        bc_fsm (i);
    }
#endif /* BATTERY_CHARGING_ENABLE */

#if RIDGE_SLAVE_ENABLE
    ridge_slave_task();
#endif /* RIDGE_SLAVE_ENABLE */

#if (defined(CCG3PA) || defined(CCG3PA2) || defined(PAG1S))
    /* Run polling tasks of CCG. */
    ccg_app_task(port);
#endif /* (defined(CCG3PA) || defined(CCG3PA2) || defined(PAG1S)) */

#if (!CCG_BACKUP_FIRMWARE)
#if ((defined(CCG5)) || (defined(CCG5C)) || (defined(CCG6)) || (defined(CCG6DF)) || (defined(CCG6SF)))
    ccg_hal_check_if_vsys_up ();
#endif /* ((defined(CCG5)) || (defined(CCG5C)) || (defined(CCG6)) || (defined(CCG6DF)) || (defined(CCG6SF))) */
#endif /* (!CCG_BACKUP_FIRMWARE) */

#ifdef PAG1S
  #if CCG_PASC_LP_ENABLE
    pd_pasc_lp_task(port);
  #endif /* CCG_PASC_LP_ENABLE */
#endif /* PAG1S */

    return true;
}

#if SYS_DEEPSLEEP_ENABLE

bool app_deepsleep_allowed(void)
{
    bool out = true;

#if (defined(CCG3PA) || defined(CCG3PA2) || defined(PAG1S))
    const dpm_status_t *dpm;
    uint8_t i = 0;

    /*
     * Deepsleep mode operation can only be supported when un-attached.
     * When attached, the references and the CSA block requires to be
     * active.
     */
    for (i = 0; i < NO_OF_TYPEC_PORTS; i++)
    {
        dpm = dpm_get_info(i);

        if (dpm->attach == true)
        {
            out = false;
            break;
        }
    }
#endif /* (defined(CCG3PA) || defined(CCG3PA2) || defined(PAG1S)) */

#ifdef PAG1S
    /*
     * In case of secondary regulator control, we will lose regulation
     * if we go into deep sleep. To avoid this, we need to monitor VBUS_IN
     * level and based on the state of it, wake up and get regulation going
     * again. If this is not done, restricting to use of SLEEP mode only.
     */
#if ((CCG_REG_SEC_CTRL) && (!CCG_PASC_LP_ENABLE))
    out = false;
#else /* !((CCG_REG_SEC_CTRL) && (!CCG_PASC_LP_ENABLE)) */
    if (out == true)
    {
        for (i = 0; i < NO_OF_TYPEC_PORTS; i++)
        {
            /*
             * For PAG1S, there is an additional sleep check required. Instead of adding
             * a seperate function, this function is overridden to do the sleep entry
             * request also from this call.
             */
#if CCG_PASC_LP_ENABLE
            if (pd_pasc_lp_is_active(i) == false)
            {
                /* We are idle and detached. We can now start the LP mode operation. */
                pd_pasc_lp_enable(i);
            }
            if (pd_pasc_lp_ds_allowed(i) == false)
            {
                out = false;
            }
#endif /* CCG_PASC_LP_ENABLE */
        }
    }
#endif /* ((CCG_REG_SEC_CTRL) && (!CCG_PASC_LP_ENABLE)) */

#endif /* PAG1S */

    return out;
}

bool app_sleep(void)
{
    bool stat = true;
    uint8_t port;

    for (port = 0; port < NO_OF_TYPEC_PORTS; port++)
    {
#if CCG_BB_ENABLE
        if (!bb_enter_deep_sleep(port))
        {
            stat = false;
            break;
        }
#endif /* CCG_BB_ENABLE */

#if (defined(CCG3PA) || defined(CCG3PA2) || defined(PAG1S))
        /*
         * Check if CCG polling tasks are not pending to be serviced and system can enter
         * low power mode.
         */
        if (ccg_app_is_idle () == false)
        {
            stat = false;
            break;
        }
#endif /* (defined(CCG3PA) || defined(CCG3PA2) || defined(PAG1S)) */

        /* Don't go to sleep while CC/SBU fault handling is pending. */
        if ((app_get_status(port)->fault_status & APP_PORT_SINK_FAULT_ACTIVE) != 0)
        {
            stat = false;
            break;
        }

#if ((DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP))
        if (!is_vdm_task_idle(port))
        {
            stat = false;
            break;
        }
#endif /* (DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP) */

#if (DP_UFP_SUPP) && (CCG_HPD_RX_ENABLE)
        /* CDT 245126 workaround: Check if HPD RX Activity timer is running.
         * If yes, don't enter deep sleep. */
        if (!is_hpd_rx_state_idle (port))
        {
            stat = false;
            break;
        }
#endif /* DP_UFP_SUPP && CCG_HPD_RX_ENABLE */

#if MUX_DELAY_EN
        /* Don't go into sleep while MUX status update is pending. */
        if (app_mux_update_req[port] != false)
        {
            stat = false;
            break;
        }
#endif /* MUX_DELAY_EN */
    }

#if ((DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP))
    if (stat)
    {
        for (port = 0; port < NO_OF_TYPEC_PORTS; port++)
        {
            /* Prepare for deep-sleep entry. */
            alt_mode_mngr_sleep(port);
        }
    }
#endif /* (DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP) */

    return stat;
}

void app_wakeup(void)
{
#if ((DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP))
    uint8_t port;

    for (port = 0; port < NO_OF_TYPEC_PORTS; port++)
    {
        alt_mode_mngr_wakeup (port);
    }
#endif /* (DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP) */
}
#endif /* SYS_DEEPSLEEP_ENABLE */

#if (CCG_BB_ENABLE != 0)
/* Alternate mode entry timeout callback function. */
static void ame_tmr_cbk(uint8_t port, timer_id_t id)
{
    (void)id;

    /* Alternate modes are reset in vdm_task_mngr_deinit(). */
    bb_enable(port, BB_CAUSE_AME_TIMEOUT);
}
#endif /* (CCG_BB_ENABLE != 0) */

#if CCG_PD_REV3_ENABLE

#if CHUNKING_NOT_SUPPORTED

void app_not_supported_sent_cb(uint8_t port, resp_status_t resp, const pd_packet_t* pkt_ptr)
{
    dpm_status_t *dpm_stat;
    /* TODO - Need to review the romfunc assignment */
    timer_cb_t romfunc = (timer_cb_t)0x1000d705;

    if (resp == CMD_SENT)
    {
        dpm_stat = dpm_get_status(port);
        if (dpm_stat->pe_fsm_state == PE_FSM_READY)
        {
            /* Send a timeout event to the policy engine by calling pe_tmr_cbk. */
            romfunc (port, PD_GENERIC_TIMER);
        }
    }
}

static void app_send_not_supported_cb (uint8_t port, timer_id_t id)
{
    dpm_pd_command_ec(port, DPM_CMD_SEND_NOT_SUPPORTED, NULL, app_not_supported_sent_cb);
}

#else /* !CHUNKING_NOT_SUPPORTED */

void extd_msg_cb(uint8_t port, resp_status_t resp, const pd_packet_t* pkt_ptr)
{
    static pd_ams_type ams_type[NO_OF_TYPEC_PORTS];
    (void)pkt_ptr;
    if(resp == RES_RCVD)
    {
        dpm_set_chunk_transfer_running(port, ams_type[port]);
    }
    if(resp == CMD_SENT)
    {
        ams_type[port] = dpm_get_info(port)->non_intr_response;
    }
}

/* Global variable used as dummy data buffer to send Chunk Request messages. */
static uint32_t gl_extd_dummy_data;

#endif /* CHUNKING_NOT_SUPPORTED */

bool app_extd_msg_handler(uint8_t port, pd_packet_extd_t *pd_pkt_p)
{
#if CHUNKING_NOT_SUPPORTED
    /*
     * If we receive any message with more than one chunk, start ChunkingNotSupportedTimer and send NOT_SUPPORTED
     * message on expiry.
     */
    if (
            (pd_pkt_p->hdr.hdr.chunked == true) &&
            (
             (pd_pkt_p->hdr.hdr.chunk_no != 0) ||
             (pd_pkt_p->hdr.hdr.data_size > ((pd_pkt_p->hdr.hdr.chunk_no + 1) * MAX_EXTD_MSG_LEGACY_LEN))
            )
       )
    {
        timer_start(port, APP_CHUNKED_MSG_RESP_TIMER, 45, app_send_not_supported_cb);

        /* Stop the PD_GENERIC_TIMER to prevent premature return to ready state. */
        timer_stop(port, PD_GENERIC_TIMER);
    }
    else
    {
#if CCG_SLN_EXTN_MSG_HANDLER_ENABLE
        /* If macro is enabled - allow handling the requests from solution space. */
        return false;
#else
        /*
         * Don't send any response to response messages. Handling here instead of in the stack so that
         * these messages can be used for PD authentication implementation.
         */
        if ((pd_pkt_p->msg != EXTD_MSG_SECURITY_RESP) && (pd_pkt_p->msg != EXTD_MSG_FW_UPDATE_RESP))
        {
            /* Send Not supported message */
            dpm_pd_command_ec(port, DPM_CMD_SEND_NOT_SUPPORTED, NULL, NULL);
        }
#endif /* CCG_HANDLE_EXT_MSG_IN_SOL */
    }

#else /* CHUNKING_NOT_SUPPORTED */
    /* If this is a chunked message which is not complete, send another chunk request. */
    if ((pd_pkt_p->hdr.hdr.chunked == true) && (pd_pkt_p->hdr.hdr.data_size >
               ((pd_pkt_p->hdr.hdr.chunk_no + 1) * MAX_EXTD_MSG_LEGACY_LEN)))
    {
        dpm_pd_cmd_buf_t extd_dpm_buf;

        extd_dpm_buf.cmd_sop = pd_pkt_p->sop;
        extd_dpm_buf.extd_type = pd_pkt_p->msg;
        extd_dpm_buf.extd_hdr.val = 0;
        extd_dpm_buf.extd_hdr.extd.chunked = true;
        extd_dpm_buf.extd_hdr.extd.request = true;
        extd_dpm_buf.extd_hdr.extd.chunk_no = pd_pkt_p->hdr.hdr.chunk_no + 1;
        extd_dpm_buf.dat_ptr = (uint8_t*)&gl_extd_dummy_data;
        extd_dpm_buf.timeout = PD_SENDER_RESPONSE_TIMER_PERIOD;

        /* Send next chunk request */
        dpm_pd_command_ec(port, DPM_CMD_SEND_EXTENDED,
                &extd_dpm_buf, extd_msg_cb);
    }
    else
    {
#if CCG_SLN_EXTN_MSG_HANDLER_ENABLE
        /* If macro is enabled - allow handling the requests from solution space. */
        return false;
#else
        /*
         * Don't send any response to response messages. Handling here instead of in the stack so that
         * these messages can be used for PD authentication implementation.
         */
        if ((pd_pkt_p->msg != EXTD_MSG_SECURITY_RESP) && (pd_pkt_p->msg != EXTD_MSG_FW_UPDATE_RESP))
        {
            /* Send Not supported message */
            dpm_pd_command_ec(port, DPM_CMD_SEND_NOT_SUPPORTED, NULL, NULL);
        }
#endif /* CCG_HANDLE_EXT_MSG_IN_SOL */
    }
#endif /* CHUNKING_NOT_SUPPORTED */

    return true;
}
#endif /* CCG_PD_REV3_ENABLE */

void app_update_bc_src_support(uint8_t port, uint8_t enable)
{
#if BATTERY_CHARGING_ENABLE && (defined(CCG5C) || defined(CCG5) || defined(CCG6))
    app_status[port].bc_12_src_disabled = (bool)(!enable);
    if (!enable)
    {
        bc_stop(port);
    }
#endif /* BATTERY_CHARGING_ENABLE && (defined(CCG5C) || defined(CCG5) || defined(CCG6)) */
}

void app_update_sys_pwr_state(uint8_t state)
{
#if BATTERY_CHARGING_ENABLE && (defined(CCG5C) || defined(CCG5) || defined(CCG6))
#if (!CCG_SINK_ONLY)
    uint8_t i = 0;
    bool    reconnect = false;

    /*
       If BC 1.2 support is enabled and the system power state changed; proceed with further checks.
       If we are currently operating as CDP, don't make any changes.
       If we are currently operating as DCP and the new power state is still S3/S4/S5, don't make any changes.
       If we are currently operating as DCP and the new power state is S0, trigger error recovery to allow
       device enumeration (only if no alternate modes are active).
     */
    if (
            (app_status[i].bc_12_src_disabled == 0) &&
            (gl_system_state != state)
       )
    {
#if CCG_PD_DUALPORT_ENABLE
        for (i = 0; i < NO_OF_TYPEC_PORTS; i++)
#endif /* CCG_PD_DUALPORT_ENABLE */
        {
            if (
                    (dpm_get_info(i)->attach) &&
                    (dpm_get_info(i)->cur_port_role == PRT_ROLE_SOURCE) &&
                    ((alt_mode_get_status(i) & 0x7F) == 0)
               )
            {
                switch (gl_system_state)
                {
                    case NB_SYS_PWR_STATE_S0:
                    case NB_SYS_PWR_STATE_S3:
                        {
                            if ((state == NB_SYS_PWR_STATE_S0) || (state == NB_SYS_PWR_STATE_S3))
                            {
                                if (bc_port_is_cdp(i) == false)
                                {
                                    reconnect = true;
                                }
                            }
                            else
                            {
                                if (bc_port_is_cdp(i) != false)
                                {
                                    reconnect = true;
                                }
                            }
                        }
                        break;

                    case NB_SYS_PWR_STATE_S4:
                    case NB_SYS_PWR_STATE_S5:
                        {
                            if (state == NB_SYS_PWR_STATE_S0)
                            {
                                reconnect = true;
                            }
                        }
                        break;

                    default:
                        break;
                }

                if (reconnect)
                {
                    dpm_typec_command (i, DPM_CMD_TYPEC_ERR_RECOVERY, NULL);
                }
            }
        }
    }
#endif /* (!CCG_SINK_ONLY) */
#endif /* BATTERY_CHARGING_ENABLE && (defined(CCG5C) || defined(CCG5) || defined(CCG6)) */

#if ((BATTERY_CHARGING_ENABLE) || (BB_RETIMER_ENABLE))
    gl_system_state = state;
#endif /* ((BATTERY_CHARGING_ENABLE) || (BB_RETIMER_ENABLE)) */

#if BB_RETIMER_ENABLE
    if(PD_GET_PTR_ICL_TGL_CFG_TBL(TYPEC_PORT_0_IDX)->icl_dual_retimer_enable != 0)
    {
        /* We're not checking if any other retimer write is pending/about to happen.
         * No code space for any fine tuned checks. So do a raw retimer write */
        retimer_set_evt(TYPEC_PORT_0_IDX, RT_EVT_UPDATE_STATUS);
    }
#if CCG_PD_DUALPORT_ENABLE
    if(PD_GET_PTR_ICL_TGL_CFG_TBL(TYPEC_PORT_1_IDX)->icl_dual_retimer_enable != 0)
    {
        retimer_set_evt(TYPEC_PORT_1_IDX, RT_EVT_UPDATE_STATUS);
    }
#endif /* CCG_PD_DUALPORT_ENABLE */
#endif /* BB_RETIMER_ENABLE */
}

#if (BATTERY_CHARGING_ENABLE && ((defined(CCG5)) || (((defined(CCG3PA) || defined(CCG3PA2)) && defined(CCG_CDP_EN)))))

/* Function to start the CCG3PA/CCG5 BC 1.2 source state machine. */
void app_bc_12_sm_start(uint8_t port)
{

#if (((defined(CCG3PA) || defined(CCG3PA2)) && defined(CCG_CDP_EN)))
    /* Proceed with CCG3PA BC1.2 CDP source state machine */
    ccg_bc_cdp_en(port);
#else /* !(((defined(CCG3PA) || defined(CCG3PA2)) && defined(CCG_CDP_EN))) */

/* Proceed with CCG5 BC1.2 Source State Machine.
 * Enable CDP is system power state is S0, otherwise enable DCP.
 */
#if CCG_HPI_ENABLE
    if (hpi_get_sys_pwr_state () == NB_SYS_PWR_STATE_S0)
#endif /* CCG_HPI_ENABLE */
    {
        ccg_bc_cdp_en(port);
    }
#if CCG_HPI_ENABLE
    else
    {
        ccg_bc_dcp_en(port);
    }
#endif /* CCG_HPI_ENABLE */
#endif /* (((defined(CCG3PA) || defined(CCG3PA2)) && defined(CCG_CDP_EN))) */
}

#endif /* (BATTERY_CHARGING_ENABLE && ((defined(CCG5)) || (((defined(CCG3PA) || defined(CCG3PA2)) && defined(CCG_CDP_EN))))) */

uint32_t get_bat_status[NO_OF_TYPEC_PORTS];

#if (ROLE_PREFERENCE_ENABLE)

/* Variable storing current preference for data role. */
volatile uint8_t app_pref_data_role[NO_OF_TYPEC_PORTS];

#if (POWER_ROLE_PREFERENCE_ENABLE)
/* Variable storing current preference for power role. */
volatile uint8_t app_pref_power_role[NO_OF_TYPEC_PORTS];
#endif /* (POWER_ROLE_PREFERENCE_ENABLE) */

/* Forward declaration of function to trigger swap operations. */
static void app_initiate_swap (uint8_t port, timer_id_t id);

static void app_role_swap_resp_cb (
        uint8_t            port,
        resp_status_t      resp,
        const pd_packet_t *pkt_ptr)
{
    app_status_t *app_stat = &app_status[port];

#if (POWER_ROLE_PREFERENCE_ENABLE)
    bool next_swap = false;
#endif /* (POWER_ROLE_PREFERENCE_ENABLE) */

    if (resp == RES_RCVD)
    {
        if (pkt_ptr->hdr.hdr.msg_type == CTRL_MSG_WAIT)
        {
            app_stat->actv_swap_count++;
            if (app_stat->actv_swap_count < APP_MAX_SWAP_ATTEMPT_COUNT)
            {
                timer_start (port, APP_INITIATE_SWAP_TIMER, app_stat->actv_swap_delay, app_initiate_swap);
            }
            else
            {
#if (POWER_ROLE_PREFERENCE_ENABLE)
                /* Swap attempts timed out. Proceed with next swap. */
                next_swap = true;
#else
                app_stat->app_pending_swaps = 0;
                app_stat->actv_swap_type  = 0;
#endif /* (POWER_ROLE_PREFERENCE_ENABLE) */
            }
        }
        else
        {
#if (POWER_ROLE_PREFERENCE_ENABLE)
            /* Swap succeeded or failed. Proceed with next swap. */
            next_swap = true;
#else
            app_stat->app_pending_swaps = 0;
            app_stat->actv_swap_type  = 0;
#endif /* (POWER_ROLE_PREFERENCE_ENABLE) */
        }
    }
    else if ((resp == CMD_FAILED) || (resp == SEQ_ABORTED) || (resp == RES_TIMEOUT))
    {
        timer_start (port, APP_INITIATE_SWAP_TIMER, app_stat->actv_swap_delay, app_initiate_swap);
    }

#if (POWER_ROLE_PREFERENCE_ENABLE)
    if (next_swap)
    {
        /* Clear the LS bit in the app_pending_swaps flag as it has completed or failed. */
        app_stat->app_pending_swaps &= (app_stat->app_pending_swaps - 1);

        app_stat->actv_swap_type  = 0;
        app_stat->actv_swap_count = 0;
        timer_start (port, APP_INITIATE_SWAP_TIMER, APP_INITIATE_DR_SWAP_TIMER_PERIOD, app_initiate_swap);
    }
#endif /* (POWER_ROLE_PREFERENCE_ENABLE) */
}

static void app_initiate_swap (uint8_t port, timer_id_t id)
{
    const dpm_status_t *dpm_stat = dpm_get_info(port);
    app_status_t *app_stat_p = app_get_status(port);
    dpm_pd_cmd_buf_t pd_cmd_buf;

    uint8_t actv_swap     = app_stat_p->actv_swap_type;
    uint8_t swaps_pending = app_stat_p->app_pending_swaps;

    /* Stop the timer that triggers swap operation. */
    timer_stop (port, APP_INITIATE_SWAP_TIMER);

    /* Nothing to do if we are not in PD contract. */
    if (!dpm_stat->contract_exist)
        return;

    if (actv_swap == 0)
    {
#if (POWER_ROLE_PREFERENCE_ENABLE)
        /* No ongoing swap operation. Pick the next pending swap from the list. */
        if ((swaps_pending & APP_VCONN_SWAP_PENDING) != 0)
        {
            actv_swap = DPM_CMD_SEND_VCONN_SWAP;
            app_stat_p->actv_swap_delay = APP_INITIATE_DR_SWAP_TIMER_PERIOD;
        }
        else
#endif /* (POWER_ROLE_PREFERENCE_ENABLE) */
        {
            if ((swaps_pending & APP_DR_SWAP_PENDING) != 0)
            {
                actv_swap = DPM_CMD_SEND_DR_SWAP;
                app_stat_p->actv_swap_delay = APP_INITIATE_DR_SWAP_TIMER_PERIOD;
            }
#if (POWER_ROLE_PREFERENCE_ENABLE)
            else
            {
                if (swaps_pending != 0)
                {
                    actv_swap = DPM_CMD_SEND_PR_SWAP;
                    app_stat_p->actv_swap_delay = APP_INITIATE_PR_SWAP_TIMER_PERIOD;
                }
            }
#endif /* (POWER_ROLE_PREFERENCE_ENABLE) */
        }

        app_stat_p->actv_swap_count = 0;
    }

    if (actv_swap != 0)
    {
        /* Check whether the selected swap is still valid. */
        switch (actv_swap)
        {
#if (POWER_ROLE_PREFERENCE_ENABLE)
            case DPM_CMD_SEND_VCONN_SWAP:
                if (dpm_stat->vconn_logical)
                {
                    app_stat_p->app_pending_swaps &= ~APP_VCONN_SWAP_PENDING;
                    actv_swap = 0;
                }
                break;
#endif /* (POWER_ROLE_PREFERENCE_ENABLE) */

            case DPM_CMD_SEND_DR_SWAP:
                /* Stop sending DR_SWAP if any alternate mode has been entered. */
                if (
                        (dpm_stat->cur_port_type == app_pref_data_role[port]) ||
                        (app_stat_p->alt_mode_entered != 0)
                   )
                {
                    app_stat_p->app_pending_swaps &= ~APP_DR_SWAP_PENDING;
                    actv_swap = 0;
                }
                break;

#if (POWER_ROLE_PREFERENCE_ENABLE)
            case DPM_CMD_SEND_PR_SWAP:
                if (dpm_stat->cur_port_role == app_pref_power_role[port])
                {
                    app_stat_p->app_pending_swaps &= ~APP_PR_SWAP_PENDING;
                    actv_swap = 0;
                }
                break;
#endif /* (POWER_ROLE_PREFERENCE_ENABLE) */

            default:
                actv_swap = 0;
                break;
        }

        if (actv_swap == 0)
        {
            /*
             * Currently selected SWAP is no longer relevant. Re-run function to identify the next swap to be
             * performed.
             */
            if (app_stat_p->app_pending_swaps != 0)
            {
                app_stat_p->actv_swap_type = 0;
                timer_start (port, APP_INITIATE_SWAP_TIMER, APP_INITIATE_DR_SWAP_TIMER_PERIOD, app_initiate_swap);
            }
        }
        else
        {
            /* Store the swap command for use in the callback. */
            app_stat_p->actv_swap_type = actv_swap;

            /* Only packet type needs to be set when initiating swap operations. */
            pd_cmd_buf.cmd_sop = SOP;

            /* Try to trigger the selected swap operation. */
            if (dpm_pd_command(port, actv_swap, &pd_cmd_buf, app_role_swap_resp_cb) != CCG_STAT_SUCCESS)
            {
                /* Retries in case of AMS failure can always be done with a small delay. */
                timer_start (port, APP_INITIATE_SWAP_TIMER, APP_INITIATE_DR_SWAP_TIMER_PERIOD, app_initiate_swap);
            }
        }
    }
}

/* This function is called at the end of a PD contract to check whether any role swaps need to be triggered. */
void app_contract_handler (uint8_t port)
{
    app_status_t *app_stat = &app_status[port];
    const dpm_status_t *dpm_stat = dpm_get_info(port);
    uint16_t delay_reqd = APP_INITIATE_PR_SWAP_TIMER_PERIOD;

#if (POWER_ROLE_PREFERENCE_ENABLE)
    /* Check if we need to go ahead with PR-SWAP. */
    if (
            (app_pref_power_role[port] == PRT_DUAL) ||
            (dpm_get_info(port)->cur_port_role == app_pref_power_role[port])
       )
    {
        app_stat->app_pending_swaps &= ~APP_PR_SWAP_PENDING;
    }
    else
    {
        /* If we are about to swap to become source, ensure VConn Swap is done as required. */
        if ((app_pref_power_role[port] == PRT_ROLE_SOURCE) && (dpm_stat->vconn_logical == 0))
        {
            app_stat->app_pending_swaps |= APP_VCONN_SWAP_PENDING;

#if (DEFER_VCONN_SRC_ROLE_SWAP)
            /* Keep the VConn SWAP role until we have finished the required tasks. */
            app_stat->keep_vconn_src = true;
#endif /* (DEFER_VCONN_SRC_ROLE_SWAP) */
        }
    }
#endif /* (POWER_ROLE_PREFERENCE_ENABLE) */

    /* Check if we need to go ahead with DR-SWAP. */
    if (
            (app_pref_data_role[port] == PRT_TYPE_DRP) ||
            (dpm_get_info(port)->cur_port_type == app_pref_data_role[port])
       )
    {
        app_stat->app_pending_swaps &= ~APP_DR_SWAP_PENDING;
    }
    else
    {
        /* DR-SWAPs need to be initiated as soon as possible. VConn swap will be triggered after DR_SWAP as needed. */
        delay_reqd = APP_INITIATE_DR_SWAP_TIMER_PERIOD;
    }

    /* Start a timer that will kick of the Swap state machine. */
    timer_start (port, APP_INITIATE_SWAP_TIMER, delay_reqd, app_initiate_swap);
}

void app_connect_change_handler (uint8_t port)
{
    /* Stop all timers used to trigger swap operations. */
    timer_stop (port, APP_INITIATE_SWAP_TIMER);

#if (POWER_ROLE_PREFERENCE_ENABLE)
    /* Assume that PR_SWAP and DR_SWAP are pending. The actual status will be updated on contract completion. */
    app_status[port].app_pending_swaps = APP_PR_SWAP_PENDING | APP_DR_SWAP_PENDING;
#else
    /* Assume that DR_SWAP is pending. The actual status will be updated on contract completion. */
    app_status[port].app_pending_swaps = APP_DR_SWAP_PENDING;
#endif /* (POWER_ROLE_PREFERENCE_ENABLE) */

    app_status[port].actv_swap_type    = 0;
    app_status[port].actv_swap_count   = 0;
}

#endif /* (ROLE_PREFERENCE_ENABLE) */

#if PD_SRC_POWER_SHARE_ENABLE

static bool     app_pwr_share_enable     = false;
static uint32_t app_sgl_port_cur_limit   = I_1P5A;
static uint8_t  app_src_extra_power_port = NO_OF_TYPEC_PORTS;
static uint8_t  app_src_pdp_default[NO_OF_TYPEC_PORTS] = {
    15
#if CCG_PD_DUALPORT_ENABLE
    ,
    15
#endif /* CCG_PD_DUALPORT_ENABLE */
};

static void app_pwr_share_recontract_cb (uint8_t port, timer_id_t id);

static void app_src_cap_change_cb(uint8_t port, resp_status_t resp, const pd_packet_t *pkt_ptr)
{
    /* If the attempt to trigger a new contract failed, queue another attempt for later. */
    if ((resp == CMD_FAILED) || (resp == SEQ_ABORTED))
    {
        timer_start(port, APP_POWER_SHARE_RECONTRACT_TIMER_ID, 10, app_pwr_share_recontract_cb);
    }
}

static void app_pwr_share_recontract_cb (uint8_t port, timer_id_t id)
{
    const dpm_status_t *dpm_stat = dpm_get_info(port);
    dpm_pd_cmd_buf_t    cmd_param;

    if ((dpm_stat->contract_exist) && (dpm_stat->cur_port_role == PRT_ROLE_SOURCE))
    {
        /* Notify the policy engine that source caps have been changed. */
        cmd_param.cmd_sop       = SOP;
        cmd_param.no_of_cmd_do  = 0;
        cmd_param.timeout       = 0;

        if (dpm_pd_command(port, DPM_CMD_SRC_CAP_CHNG, &cmd_param, app_src_cap_change_cb) !=
                CCG_STAT_SUCCESS)
        {
            timer_start(port, APP_POWER_SHARE_RECONTRACT_TIMER_ID, 10, app_pwr_share_recontract_cb);
        }
    }
}

static void app_update_src_caps(uint8_t port, uint32_t max_cur)
{
    dpm_status_t           *dpm_stat = dpm_get_status(port);
    pd_do_t                 src_cap;
    uint8_t                 cur_lvl = DPM_CMD_SET_RP_1_5A;
    uint8_t                 pdp_val = 7;                        /* 5V@1.5A ==> 7.5 W */
    uint8_t                 pdomask;

    /* Start with previous PDO mask value. */
    pdomask = dpm_stat->src_pdo_mask;

    /* Update Rp to reflect the power reserved for this port. */
    if (max_cur >= I_3A)
    {
        cur_lvl = DPM_CMD_SET_RP_3A;
        pdp_val = 15;                       /* 5V@3A ==> 15 W. */
    }
    if (max_cur < I_1P5A)
    {
        cur_lvl = DPM_CMD_SET_RP_DFLT;
        pdp_val = 4;                        /* 5V@900mA = 4.5 W. */
    }

    pdomask = ((pdomask & 0x80) | 1);
    src_cap = dpm_stat->src_pdo[0];
    src_cap.fixed_src.max_current = max_cur;

    /* Directly update the DPM-STATUS structures. */
    dpm_stat->src_cur_level = cur_lvl;
    dpm_stat->src_pdo[0]    = src_cap;
    dpm_stat->src_pdo_mask  = pdomask;
    dpm_stat->ext_src_cap[CCG_PD_EXT_SRCCAP_PDP_INDEX] = pdp_val;

    if ((dpm_stat->contract_exist) && (dpm_stat->cur_port_role == PRT_ROLE_SOURCE))
    {
        /* Start timer to trigger re-contract with changed power allocation. */
        timer_start (port, APP_POWER_SHARE_RECONTRACT_TIMER_ID, 10, app_pwr_share_recontract_cb);
    }
}

static void app_power_share_update(uint8_t port)
{
    const dpm_status_t *dpm_stat = dpm_get_info(port);

    /* Skip actions when power sharing is disabled. */
    if (!app_pwr_share_enable)
    {
        return;
    }

    /*
     * If we are source, sink is reporting capability mismatch, and spare power is available; increase the
     * power allocation for this port.
     */
    if (
            (dpm_stat->contract_exist) &&
            (dpm_stat->cur_port_role == PRT_ROLE_SOURCE)
       )
    {
        if (
                (dpm_stat->src_rdo.rdo_fix_var.cap_mismatch) &&
                (app_src_extra_power_port == NO_OF_TYPEC_PORTS)
           )
        {
            /* Allocate the power reserve to this port. */
            app_src_extra_power_port = port;

            /* Update PDOs and make a new contract. */
            app_update_src_caps (port, I_3A);
        }
    }
    else
    {
        if (app_src_extra_power_port == port)
        {
            /* Free up the extra power that was allocated to this port. */
            app_src_extra_power_port = NO_OF_TYPEC_PORTS;

            /* Update PDOs and make a new contract. */
            app_update_src_caps (port, app_sgl_port_cur_limit);
        }
    }
}

void app_power_share_init(void)
{
    const pd_port_config_t *conf_p = get_pd_port_config(0);
    pd_do_t src_pdo;

    /* Nothing to do on a single port device. */
    if (NO_OF_TYPEC_PORTS == 1)
    {
        return;
    }

    app_pwr_share_enable = true;
    app_sgl_port_cur_limit = I_1P5A;

    /* Pick the default power allocation per port from the config table. */
    src_pdo.val = conf_p->src_pdo_list[0];
    if (src_pdo.fixed_src.max_current < I_1P5A)
    {
        app_sgl_port_cur_limit = I_0P9A;
    }

    /* Store the original PDP values for each port. */
    app_src_pdp_default[0] = dpm_get_info(0)->ext_src_cap[CCG_PD_EXT_SRCCAP_PDP_INDEX];
    app_src_pdp_default[1] = dpm_get_info(1)->ext_src_cap[CCG_PD_EXT_SRCCAP_PDP_INDEX];

    if (app_src_extra_power_port >= NO_OF_TYPEC_PORTS)
    {
        /* Initially allocate limited power per port. */
        app_update_src_caps (0, app_sgl_port_cur_limit);
        app_update_src_caps (1, app_sgl_port_cur_limit);
    }
}

#endif /* PD_SRC_POWER_SHARE_ENABLE */

#if RIDGE_SLAVE_ENABLE
static void app_check_usb_supp(uint8_t port)
{
    const dpm_status_t *dpm_stat = dpm_get_info(port);

    /* Check port partner's USB capabilities */
    if (
#if (!CCG_SINK_ONLY)
         ((dpm_stat->cur_port_role == PRT_ROLE_SOURCE)                 &&
          ((dpm_stat->src_rdo.rdo_gen.usb_comm_cap == false)           ||
          (dpm_stat->cur_src_pdo[0].fixed_src.usb_comm_cap == false))) ||
         ((dpm_stat->cur_port_role == PRT_ROLE_SINK)                   &&
          ((dpm_stat->snk_rdo.rdo_gen.usb_comm_cap == false)           ||
          (dpm_stat->src_cap_p->dat[0].fixed_src.usb_comm_cap == false)))
#else
          ((dpm_stat->snk_rdo.rdo_gen.usb_comm_cap == false)           ||
          (dpm_stat->src_cap_p->dat[0].fixed_src.usb_comm_cap == false))
#endif /* (!CCG_SINK_ONLY) */
       )
    {
        app_status[port].usb2_supp = false;
        app_status[port].usb3_supp = false;
    }
}
#endif /* RIDGE_SLAVE_ENABLE */

static uint8_t gl_app_previous_polarity[NO_OF_TYPEC_PORTS];

static bool gl_app_hard_rst_sent_rcvd[NO_OF_TYPEC_PORTS] = {
    false
#if CCG_PD_DUALPORT_ENABLE
    ,
    false
#endif /* CCG_PD_DUALPORT_ENABLE */
};

#if (MUX_DELAY_EN)
void app_hard_rst_retimer_cbk (uint8_t port, timer_id_t id)
{
    app_event_handler(port, APP_EVT_HR_SENT_RCVD_DEFERRED, NULL);
}
#endif /* (MUX_DELAY_EN) */

void app_event_handler(uint8_t port, app_evt_t evt, const void* dat)
{
    const app_req_status_t* result;
    const pd_contract_info_t* contract_status;
    bool  skip_soln_cb = false;
    bool  hardreset_cplt = false;
    bool  typec_only = false;
    dpm_status_t *dpm_stat = dpm_get_status(port);

#if CCG_PD_REV3_ENABLE
#if HOST_ALERT_MSG_ENABLE
    pd_do_t alert_ado;
#endif /* HOST_ALERT_MSG_ENABLE */
#endif /* CCG_PD_REV3_ENABLE */

    switch(evt)
    {
        case APP_EVT_TYPEC_STARTED:
#if (defined(CCG3PA) || defined(CCG3PA2) || defined(PAG1S))
#if (POWER_BANK == 1)
            /*
             * In power bank case, if device is powered by external VDDD (i.e not
             * in dead battery), disable internal VBUS regulator.
             */
            if (dpm_stat->dead_bat == false)
            {
                pd_hal_disable_vreg (port);
            }
#endif /* (POWER_BANK == 1) */
#endif /* (defined(CCG3PA) || defined(CCG3PA2) || defined(PAG1S)) */

            /* Initialize the MUX to its default settings (isolate). */
            mux_ctrl_init (port);
#if ((DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP))
            app_status[port].vdm_prcs_failed = false;
#endif /* ((DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP)) */
#if BB_RETIMER_ENABLE
            if (dpm_stat->dead_bat == true)
            {
                /* Enable the retimer power and release it from reset. */
                retimer_enable(port, false);
            }
            else
            {
                /* Make sure the retimer is disabled by default. */
                retimer_disable(port, false);
            }
#endif /* BB_RETIMER_ENABLE */
            break;

        case APP_EVT_TYPEC_ATTACH:
#if CCG_REV3_HANDLE_BAD_SINK
            /* Start bad sink timer */
            if (!(timer_is_running (port, APP_BAD_SINK_TIMEOUT_TIMER) && (true == gl_bad_sink_apdo_sel[port])))
            {
                timer_stop(port, APP_BAD_SINK_TIMEOUT_TIMER);
                timer_start(port, APP_BAD_SINK_TIMEOUT_TIMER, APP_BAD_SINK_TIMEOUT_TIMER_PERIOD, app_bad_sink_timeout_cbk);
            }
#if (CCG_LOAD_SHARING_ENABLE)
            if (true == gl_bad_device[port])
            {
                ccg_power_throttle_set_stop_ls(port, false);
                ccg_power_throttle_set_stop_power_throttle(port, false);
                gl_bad_device[port] = false;
            }
#endif /* (CCG_LOAD_SHARING_ENABLE) */
#endif /* CCG_REV3_HANDLE_BAD_SINK */

#if RIDGE_SLAVE_ENABLE
            /* Set USB2/USB3 connection bits to 1 by default. */
            app_status[port].usb2_supp = true;
            app_status[port].usb3_supp = true;
#endif /* RIDGE_SLAVE_ENABLE */

            /* This will also enable the USB (DP/DM) MUX where required. */
            set_mux (port, MUX_CONFIG_SS_ONLY, 0);

            /* Clear all fault counters if we have seen a change in polarity from previous connection. */
            if (dpm_stat->polarity != gl_app_previous_polarity[port])
            {
                fault_handler_clear_counts (port);
            }
            gl_app_previous_polarity[port] = dpm_stat->polarity;
            break;

        case APP_EVT_CONNECT:
#if ((DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP))
            app_status[port].vdm_prcs_failed = false;
#endif /* ((DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP)) */
#if (!CCG_CBL_DISC_DISABLE)
            app_status[port].cbl_disc_id_finished = false;
            app_status[port].disc_cbl_pending = false;
#endif /* (!CCG_CBL_DISC_DISABLE) */

#if DEBUG_ACCESSORY_SNK_ENABLE
            if ((dpm_stat->attached_dev == DEV_DBG_ACC) && (dpm_stat->cur_port_role == PRT_ROLE_SINK))
            {
#if RIDGE_SLAVE_ENABLE
                /* Enable USB2/3 bits when connected to a Debug Accessory. */
                app_status[port].usb2_supp = true;
                app_status[port].usb3_supp = true;
#endif /* RIDGE_SLAVE_ENABLE */

                app_status[port].debug_acc_attached = true;
                set_mux (port, MUX_CONFIG_SS_ONLY, 0);
            }
#endif /* DEBUG_ACCESSORY_SNK_ENABLE */

#if (CCG_BB_ENABLE != 0)
            /* Enable the AME timer on attach if in sink mode. */
            if (gl_dpm_port_type[port] == PRT_TYPE_UFP)
            {
                if (bb_bind_to_port(port) == CCG_STAT_SUCCESS)
                {
                    /* Start the AME timer in any case as the MUX needs to be setup in the callback in some cases. */
                    timer_start(port, APP_AME_TIMEOUT_TIMER, APP_AME_TIMEOUT_TIMER_PERIOD, ame_tmr_cbk);

                    /* Leave self power status flag cleared as we do not have a PD contract in place. */
                    bb_update_self_pwr_status (port, 0);
                }
            }
#endif /* (CCG_BB_ENABLE != 0) */

#if (ROLE_PREFERENCE_ENABLE)
            app_connect_change_handler (port);
#endif /* (ROLE_PREFERENCE_ENABLE) */
            break;

        case APP_EVT_HARD_RESET_COMPLETE:
            hardreset_cplt = true;
            /* Intentional fall-through. */

        case APP_EVT_HR_SENT_RCVD_DEFERRED:
        case APP_EVT_HARD_RESET_SENT:
        case APP_EVT_PE_DISABLED:
            typec_only = ((dpm_stat->pd_connected == false) || (evt == APP_EVT_PE_DISABLED));
            /* Intentional fall-through. */

        case APP_EVT_HARD_RESET_RCVD:
        case APP_EVT_VBUS_PORT_DISABLE:
        case APP_EVT_DISCONNECT:
        case APP_EVT_TYPE_C_ERROR_RECOVERY:

#if (!CCG_CBL_DISC_DISABLE)
            app_status[port].cbl_disc_id_finished = false;
            app_status[port].disc_cbl_pending = false;
#endif /* (!CCG_CBL_DISC_DISABLE) */
#if CCG_REV3_HANDLE_BAD_SINK
            if ((evt == APP_EVT_DISCONNECT) || (evt == APP_EVT_VBUS_PORT_DISABLE))
            {
                /* Stop bad sink timer and clear bad sink status. */
                if (!(timer_is_running (port, APP_BAD_SINK_TIMEOUT_TIMER) && (true == gl_bad_sink_apdo_sel[port])))
                {
                    timer_stop(port, APP_BAD_SINK_TIMEOUT_TIMER);
                }
                gl_bad_sink_timeout_status[port] = false;
                gl_bad_sink_pd_status[port] = false;
#if (CCG_LOAD_SHARING_ENABLE && CCG_REV3_HANDLE_BAD_SINK)
                if (false == gl_bad_sink_apdo_sel[port])
                {
                    gl_bad_device[port] = false;
                }
#endif /* (CCG_LOAD_SHARING_ENABLE && CCG_REV3_HANDLE_BAD_SINK) */
            }
#if (CCG_TEMP_BASED_VOLTAGE_THROTTLING || CCG_VIN_BASED_VOLTAGE_THROTTLING || CCG_LOAD_SHARING_ENABLE)
            if((evt == APP_EVT_DISCONNECT) && (true == ccg_power_throttle_get_power_throttle_cmd_pending(port)))
            {
                ccg_power_throttle_set_power_throttle_renegotiation_complete(port, true);
            }
#endif /* (CCG_TEMP_BASED_VOLTAGE_THROTTLING || CCG_VIN_BASED_VOLTAGE_THROTTLING || CCG_LOAD_SHARING_ENABLE) */


#endif /* CCG_REV3_HANDLE_BAD_SINK */
#if (defined(CCG3PA) || defined(CCG3PA2) || defined(PAG1S))
#if (POWER_BANK == 1)
            /*
             * In power bank case, if device is powered by external VDDD, VDDD gets
             * shorted to VBUS_IN line. This shall result connecting VDDD to the
             * Type-C VBUS line. This also includes cases where we start as dead
             * dead battery device and then get charged. So if any time VBUS has to
             * be removed in course of PD / Type-C state machine, ensure that internal
             * VBUS regulator is disabled. In event of dead battery, this shall lead
             * to device reset. This is the safest recovery path. CDT 276535.
             *
             * This code can be removed if the VBATT monitoring can be done
             * continously. But this code can still be in place to avoid any
             * corner case handling.
             */

            /* Do this only on disconnect and type-C error recovery. */
            if ((evt == APP_EVT_DISCONNECT) || (evt == APP_EVT_TYPE_C_ERROR_RECOVERY))
            {
                pd_hal_disable_vreg(port);
            }
#endif /* (POWER_BANK == 1) */
#endif /* (defined(CCG3PA) || defined(CCG3PA2) || defined(PAG1S)) */

            if (
                    (evt == APP_EVT_HARD_RESET_SENT) ||
                    (evt == APP_EVT_HARD_RESET_RCVD) ||
                    (evt == APP_EVT_HR_SENT_RCVD_DEFERRED)
               )
            {
                gl_app_hard_rst_sent_rcvd[port] = true;

#if MUX_DELAY_EN
                if (app_get_status(port)->is_mux_busy == false)
#endif /* MUX_DELAY_EN */
                {
#if BB_RETIMER_ENABLE
                    /* Disable and then re-enable the retimer with a delay. */
                    retimer_status_update (port, 0x00, 0);
                    retimer_disable(port, false);
                    retimer_enable(port, true);
#endif /* BB_RETIMER_ENABLE */

#if RIDGE_SLAVE_ENABLE
                    /* Drop all previous MUX status updates and update the status as 0 (no data connection). */
                    set_mux (port, MUX_CONFIG_DEINIT, 0);
#endif /* RIDGE_SLAVE_ENABLE */

#if BB_RETIMER_ENABLE
                    app_status[port].skip_mux_config = true;
#endif /* BB_RETIMER_ENABLE */
                }
#if MUX_DELAY_EN
                else
                {
                    /* Run MUX delay timer */
                    timer_start(port, ICL_HARD_RST_TIMER,
                            timer_get_count(port, APP_MUX_DELAY_TIMER) + 2,
                            app_hard_rst_retimer_cbk);
                    break;
                }
#endif /* MUX_DELAY_EN */
            }

#if ((DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP))

            if (evt == APP_EVT_DISCONNECT)
            {
#if DEBUG_ACCESSORY_SNK_ENABLE
                /* Mark debug accessory detached. */
                app_status[port].debug_acc_attached = false;
#endif /* DEBUG_ACCESSORY_SNK_ENABLE */
            }

#if (CCG_USB4_SUPPORT_ENABLE)
            app_status[port].usb4_active = false;
            app_status[port].usb4_data_rst_cnt = 0;
#endif /* CCG_USB4_SUPPORT_ENABLE */

#if BB_RETIMER_ENABLE
            if (!typec_only)
            {
                app_status[port].skip_mux_config = true;
            }
#endif /* BB_RETIMER_ENABLE */

            vdm_task_mngr_deinit (port);

#if ((defined(CCG5)) || (defined(CCG5C)) || (defined(CCG6)) || (defined(CCG6DF)) || (defined(CCG6SF)))
            timer_stop (port, APP_CBL_DISC_TRIGGER_TIMER);
#endif /* ((defined(CCG5)) || (defined(CCG5C)) || (defined(CCG6)) || (defined(CCG6DF)) || (defined(CCG6SF))) */

#endif /* (DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP) */

#if BB_RETIMER_ENABLE
            app_status[port].skip_mux_config = false;
#endif /* BB_RETIMER_ENABLE */

#if PD_SRC_POWER_SHARE_ENABLE
            /* If this port had the excess power allocated, free it up. */
            app_power_share_update (port);
#endif /* PD_SRC_POWER_SHARE_ENABLE */

            if (hardreset_cplt)
            {
#if BB_RETIMER_ENABLE
                if(dpm_stat->cur_port_role == PRT_ROLE_SINK)
                {
                    set_retimer_status(port, true);
                }
                app_status[port].skip_mux_config = false;
#endif /* BB_RETIMER_ENABLE */

                set_mux (port, MUX_CONFIG_SS_ONLY, 0);
            }
            else
            {
                /*
                 * Isolate the data lines if this is a PD connection.
                 */
                if (!typec_only)
                {
#if !RIDGE_SLAVE_ENABLE
                    set_mux (port, MUX_CONFIG_ISOLATE, 0);
#endif /* !RIDGE_SLAVE_ENABLE */
                    timer_stop (port, APP_AME_TIMEOUT_TIMER);

#if BB_RETIMER_ENABLE
#if (!CCG_SINK_ONLY)
                    if (
                            (!gl_app_hard_rst_sent_rcvd[port])
#if (!CCG_BACKUP_FIRMWARE)
                            &&
                            (dpm_stat->typec_fsm_state != TYPEC_FSM_TRY_WAIT_SNK)
#endif /* (!CCG_BACKUP_FIRMWARE) */
                       )
                    {
                        app_status[port].retimer_dis_req = true;
                    }
#endif /* (!CCG_SINK_ONLY) */
#endif /* BB_RETIMER_ENABLE */

#if RIDGE_SLAVE_ENABLE
                    set_mux(port, MUX_CONFIG_RIDGE_CUSTOM, NO_DATA);
#endif /* RIDGE_SLAVE_ENABLE */
                }
            }
#if (VBUS_OCP_ENABLE | VBUS_SCP_ENABLE | VBUS_OVP_ENABLE| VBUS_UVP_ENABLE | ICL_OCP_ENABLE)
            if(evt == APP_EVT_TYPE_C_ERROR_RECOVERY)
            {
                /* Clear port-in-fault flag if all fault counts are within limits. */
                if (!app_port_fault_count_exceeded(port))
                {
                    if ((app_get_status(port)->fault_status & APP_PORT_DISABLE_IN_PROGRESS) == 0)
                    {
                        dpm_clear_fault_active(port);
                    }
                }
            }
#endif /* VBUS_OCP_ENABLE | VBUS_SCP_ENABLE | VBUS_OVP_ENABLE| VBUS_UVP_ENABLE */

            if ((evt == APP_EVT_DISCONNECT) || (evt == APP_EVT_VBUS_PORT_DISABLE))
            {
#if RIDGE_SLAVE_ENABLE && BB_RETIMER_ENABLE
                ridge_slave_set_debug(port, 0x00);
#endif /*  RIDGE_SLAVE_ENABLE && BB_RETIMER_ENABLE */

#if CCG_BACKUP_FIRMWARE
#if BB_RETIMER_ENABLE
                /* Turn the retimer off after it's been written to */
                retimer_status_update (port, 0x00, 0);
                retimer_disable (port, (gl_system_state != NB_SYS_PWR_STATE_S0));
#endif /* BB_RETIMER_ENABLE */
#endif /* CCG_BACKUP_FIRMWARE */

#if REGULATOR_REQUIRE_STABLE_ON_TIME
                /* Disable the regulator on port disconnect */
                REGULATOR_DISABLE(port);
#endif /* REGULATOR_REQUIRE_STABLE_ON_TIME */

                /* Cleanup the PD block states on disconnect. */
                pd_hal_cleanup(port);

#if VCONN_OCP_ENABLE
                /* Clear the VConn fault status. */
                app_status[port].fault_status &= ~APP_PORT_VCONN_FAULT_ACTIVE;
#endif /* VCONN_OCP_ENABLE */

#if CCG_BB_ENABLE
                if (evt == APP_EVT_DISCONNECT)
                {
                    bb_disable (port, true);

                    /* Clear self power status flag cleared on disconnect. */
                    bb_update_self_pwr_status (port, 0);
                }
#endif /* CCG_BB_ENABLE */
            }

#if ((!CCG_SINK_ONLY) || (ROLE_PREFERENCE_ENABLE))
            if (
                    (evt == APP_EVT_HARD_RESET_COMPLETE) ||
                    (evt == APP_EVT_TYPE_C_ERROR_RECOVERY) ||
                    (evt == APP_EVT_DISCONNECT)
               )
            {
#if (!CCG_SINK_ONLY)
                gl_app_hard_rst_sent_rcvd[port] = false;
#endif /* (!CCG_SINK_ONLY) */

#if (ROLE_PREFERENCE_ENABLE)
                /* Stop the DR-Swap and PR-Swap trigger timers.  Assume that
                 * PR_SWAP and DR_SWAP are pending. The actual status will be
                 * updated on contract completion.
                 */
                app_connect_change_handler (port);
#endif /* (ROLE_PREFERENCE_ENABLE) */
            }
#endif /* ((!CCG_SINK_ONLY) || (ROLE_PREFERENCE_ENABLE)) */

#if APP_PPS_SINK_SUPPORT
            /* Make sure the PPS re-negotiation task is stopped. */
            app_pps_sink_disable (port);
#endif /* APP_PPS_SINK_SUPPORT */

#if APP_FRS_RX_ENABLE
            gl_frs_in_prog[port] = false;
#endif /* APP_FRS_RX_ENABLE */
            break;

#if (!CCG_CBL_DISC_DISABLE)
        case APP_EVT_EMCA_NOT_DETECTED:
        case APP_EVT_EMCA_DETECTED:
            app_status[port].cbl_disc_id_finished = true;
            app_status[port].disc_cbl_pending = false;
            app_status[port].vdm_prcs_failed = false;

#if (DEFER_VCONN_SRC_ROLE_SWAP)
#if DFP_ALT_MODE_SUPP
            /* If alt. mode state machine is inactive or idle, we can allow VConn swaps to proceed. */
            if (is_vdm_task_idle(port))
            {
                app_status[port].keep_vconn_src = false;
            }
#endif /* DFP_ALT_MODE_SUPP */
#endif /* (DEFER_VCONN_SRC_ROLE_SWAP) */
            break;
#endif /* (!CCG_CBL_DISC_DISABLE) */

        case APP_EVT_DR_SWAP_COMPLETE:
            result = (const app_req_status_t*)dat ;
            if(*result == REQ_ACCEPT)
            {
#if (ROLE_PREFERENCE_ENABLE)
                app_status[port].app_pending_swaps &= ~APP_DR_SWAP_PENDING;
                if (app_status[port].actv_swap_type == DPM_CMD_SEND_DR_SWAP)
                {
                    timer_stop (port, APP_INITIATE_SWAP_TIMER);
                    app_contract_handler (port);
                }
#endif /* (ROLE_PREFERENCE_ENABLE) */

#if ((RIDGE_SLAVE_ENABLE) || (BB_RETIMER_ENABLE))
                /* Check if source and sink ports supports usb communication */
                app_check_usb_supp(port);
                /* Update Retimer/SoC data role bit */
                ridge_update_dr(port);
#endif /* (RIDGE_SLAVE_ENABLE) || (BB_RETIMER_ENABLE) */

#if ((DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP))
#if CCG_USB4_SUPPORT_ENABLE
                if (app_status[port].usb4_active != false)
                {
                    /* Run VDM manager */
                    app_status[port].vdm_task_en = false;
                    enable_vdm_task_mngr(port);
                }
                else
#endif /* CCG_USB4_SUPPORT_ENABLE */
                {
                    /* Device data role changed. Reset alternate mode layer. */
                    alt_mode_layer_reset(port);
                }
#endif /* (DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP) */

#if (CCG_BB_ENABLE != 0)
                /* Start tAME Timer to enable BB functionality */
                if (gl_dpm_port_type[port] == PRT_TYPE_UFP)
                {
                    if (bb_bind_to_port(port) == CCG_STAT_SUCCESS)
                    {
                        timer_start(port, APP_AME_TIMEOUT_TIMER , APP_AME_TIMEOUT_TIMER_PERIOD, ame_tmr_cbk);
                    }
                }
                else
                {
                    timer_stop (port, APP_AME_TIMEOUT_TIMER);
                }
#endif /* (CCG_BB_ENABLE != 0) */
            }
            break;

#if (!CCG_BACKUP_FIRMWARE)
        case APP_EVT_VENDOR_RESPONSE_TIMEOUT:
            /* If the APP layer is going to retry the VDM, do not send the event. */
            if (app_status[port].vdm_retry_pending)
                skip_soln_cb = true;
            break;
#endif /* (!CCG_BACKUP_FIRMWARE) */
        case APP_EVT_BAD_SINK_APDO_SEL:
#if CCG_REV3_HANDLE_BAD_SINK
            gl_bad_sink_apdo_sel[port] = true;
#endif /* CCG_REV3_HANDLE_BAD_SINK */
            break;

        case APP_EVT_PD_CONTRACT_NEGOTIATION_COMPLETE:
            contract_status = (pd_contract_info_t*)dat;

            /* Set VDM version based on active PD revision. */
#if CCG_PD_REV3_ENABLE
            if (dpm_stat->spec_rev_sop_live >= PD_REV3)
            {
                app_status[port].vdm_version = STD_VDM_VERSION_REV3;
            }
            else
#endif /* CCG_PD_REV3_ENABLE */
            {
                app_status[port].vdm_version = STD_VDM_VERSION_REV2;
            }

            if ((contract_status->status == PD_CONTRACT_NEGOTIATION_SUCCESSFUL) ||
                    (contract_status->status == PD_CONTRACT_CAP_MISMATCH_DETECTED))
            {
#if CCG_REV3_HANDLE_BAD_SINK
                /*
                 * Stop bad sink timer, but retain bad sink status until port
                 * detach or disable.
                 */
                timer_stop(port, APP_BAD_SINK_TIMEOUT_TIMER);
                if (true == gl_bad_sink_apdo_sel[port])
                {
                    gl_bad_sink_apdo_sel[port] = false;
                }   
#endif /* CCG_REV3_HANDLE_BAD_SINK */

#if PD_SRC_POWER_SHARE_ENABLE
                app_power_share_update (port);
#endif /* PD_SRC_POWER_SHARE_ENABLE */

#if (ROLE_PREFERENCE_ENABLE)
                app_contract_handler (port);
#endif /* (ROLE_PREFERENCE_ENABLE) */

#if ((DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP))
                /*
                 * Contract established.  Enable VDM task manager for Alt. Mode support.
                 * This function will have no effect if the Alt. Modes are already running.
                 */
                if (
                        (gl_dpm_port_type[port] == PRT_TYPE_UFP) ||
                        (app_status[port].vdm_prcs_failed == false)
                   )
                {
                    enable_vdm_task_mngr(port);
                }
#endif /* (DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP) */
            }

#if (CCG_BB_ENABLE != 0)
            /* Now that there is a PD contract in place, we can set the self powered status flag. */
            bb_update_self_pwr_status (port, 1);

            if (
                    (contract_status->status != PD_CONTRACT_NEGOTIATION_SUCCESSFUL) &&
                    (dpm_get_info(port)->cur_port_role == PRT_ROLE_SINK) &&
                    (gl_dpm_port_type[port] == PRT_TYPE_UFP)
               )
            {
                bb_enable(port, BB_CAUSE_PWR_FAILURE);
            }
#endif /* (CCG_BB_ENABLE != 0) */

#if (CCG_TEMP_BASED_VOLTAGE_THROTTLING || CCG_VIN_BASED_VOLTAGE_THROTTLING || CCG_LOAD_SHARING_ENABLE)
            if(true == ccg_power_throttle_get_power_throttle_cmd_pending(port))
            {
                 ccg_power_throttle_set_power_throttle_renegotiation_complete(port, true);
            }
#endif /* (CCG_TEMP_BASED_VOLTAGE_THROTTLING || CCG_VIN_BASED_VOLTAGE_THROTTLING || CCG_LOAD_SHARING_ENABLE) */

#if (DEFER_VCONN_SRC_ROLE_SWAP)
            /* If we are not DFP, we can now allow the port partner to become VConn Source. */
            if (
                    (gl_dpm_port_type[port] == PRT_TYPE_UFP)
#if ROLE_PREFERENCE_ENABLE
                    &&
                    (app_pref_data_role[port] != PRT_TYPE_DFP)
#endif /* ROLE_PREFERENCE_ENABLE */
               )
            {
                app_status[port].keep_vconn_src = false;
            }
#endif /* (DEFER_VCONN_SRC_ROLE_SWAP) */

#if RIDGE_SLAVE_ENABLE
            /* Set USB2/USB3 connection bits to 1 by default*/
            app_status[port].usb2_supp = true;
            app_status[port].usb3_supp = true;

            /* Check if source and sink ports supports usb communication */
            app_check_usb_supp(port);
#endif /* RIDGE_SLAVE_ENABLE */
            break;

#if CCG_PD_REV3_ENABLE
        case APP_EVT_HANDLE_EXTENDED_MSG:
#if (CCG_HPI_PD_ENABLE)
            /* Handle the extended message locally if forwarding to EC is not enabled. */
            if (hpi_is_extd_msg_ec_ctrl_enabled (port) == false)
#endif
            {
                if(!(app_extd_msg_handler(port, (pd_packet_extd_t *)dat)))
                {
                    skip_soln_cb  = false;
                }
                else
                {
                    skip_soln_cb  = true;
                }
            }
            break;
#if HOST_ALERT_MSG_ENABLE
        case APP_EVT_ALERT_RECEIVED:
            /* Respond to ALERT message only if there is the number of object is one. */
            if (((pd_packet_t*)dat)->len == 1)
            {
                alert_ado = ((pd_packet_t*)dat)->dat[0];
                if(alert_ado.ado_alert.bat_status_change == false)
                {
                    dpm_pd_command(port, DPM_CMD_GET_STATUS, NULL, NULL);
                }
                else
                {
                    uint8_t i = alert_ado.ado_alert.fixed_bats |
                        (alert_ado.ado_alert.hot_swap_bats << 4);
                    dpm_pd_cmd_buf_t cmd;

                    /* Identify the first battery for which the change is intended. */
                    get_bat_status[port] = 0;
                    while ((i != 0) && ((i & 0x01) == 0))
                    {
                        get_bat_status[port]++;
                        i >>= 1;
                    }

                    cmd.cmd_sop = SOP;
                    cmd.extd_hdr.val = 0x1;
                    cmd.timeout = PD_SENDER_RESPONSE_TIMER_PERIOD;
                    cmd.extd_type = EXTD_MSG_GET_BAT_STATUS;
                    cmd.dat_ptr = (uint8_t*)&get_bat_status[port];
                    dpm_pd_command(port, DPM_CMD_SEND_EXTENDED, &cmd, NULL);
                }
            }
            break;
#endif /* HOST_ALERT_MSG_ENABLE */
#endif /* CCG_PD_REV3_ENABLE */

        case APP_EVT_TYPEC_ATTACH_WAIT:
#if REGULATOR_REQUIRE_STABLE_ON_TIME
            REGULATOR_ENABLE(port);
#endif /* REGULATOR_REQUIRE_STABLE_ON_TIME */

#if BB_RETIMER_ENABLE
            retimer_enable(port, false);
#endif /* BB_RETIMER_ENABLE */

#if CCG_PASC_LP_ENABLE
            pd_pasc_lp_disable(port);
#endif /* CCG_PASC_LP_ENABLE */
            break;

        case APP_EVT_TYPEC_ATTACH_WAIT_TO_UNATTACHED:
#if REGULATOR_REQUIRE_STABLE_ON_TIME
            REGULATOR_DISABLE(port);
#endif /* REGULATOR_REQUIRE_STABLE_ON_TIME */

#if BB_RETIMER_ENABLE
            /* If we are in Unattached states, turn the retimer off after it's been written to */
            if (
                    (dpm_stat->typec_fsm_state == TYPEC_FSM_UNATTACHED_SNK)
#if (!CCG_SINK_ONLY)
                    ||
                    (dpm_stat->typec_fsm_state == TYPEC_FSM_UNATTACHED_SRC)
#endif /* (!CCG_SINK_ONLY) */
               )
            {
                app_status[port].retimer_dis_req = true;
                set_mux(port, MUX_CONFIG_RIDGE_CUSTOM, NO_DATA);
            }
#endif /* BB_RETIMER_ENABLE */
            break;

#if ((POWER_ROLE_PREFERENCE_ENABLE) || (PD_SRC_POWER_SHARE_ENABLE))
        case APP_EVT_PR_SWAP_COMPLETE:
#if (POWER_ROLE_PREFERENCE_ENABLE)
            app_status[port].app_pending_swaps &= ~APP_PR_SWAP_PENDING;
            if (app_status[port].actv_swap_type == DPM_CMD_SEND_PR_SWAP)
            {
                timer_stop (port, APP_INITIATE_SWAP_TIMER);
                app_contract_handler (port);
            }
#endif /* (POWER_ROLE_PREFERENCE_ENABLE) */
#if PD_SRC_POWER_SHARE_ENABLE
            app_power_share_update (port);
#endif /* PD_SRC_POWER_SHARE_ENABLE */
            break;
#endif /* ((POWER_ROLE_PREFERENCE_ENABLE) || (PD_SRC_POWER_SHARE_ENABLE)) */

#if (!CCG_SINK_ONLY)
#if (DEFER_VCONN_SRC_ROLE_SWAP)
        case APP_EVT_ALT_MODE:
#if DFP_ALT_MODE_SUPP
            /* We can clear the keep VConn source flag once the alternate mode state machine has become idle. */
            if (is_vdm_task_idle (port))
            {
                app_status[port].keep_vconn_src = false;
            }
#endif /* DFP_ALT_MODE_SUPP */
            break;
#endif /* (DEFER_VCONN_SRC_ROLE_SWAP) */
#endif /* (!CCG_SINK_ONLY) */

        case APP_EVT_DATA_RESET_ACCEPTED:
            /* Need to reset alternate modes as part of Data_Reset. No status update is to be provided. */
            alt_mode_layer_reset (port);

#if (!CCG_CBL_DISC_DISABLE)
            app_status[port].cbl_disc_id_finished = false;
#endif /* (!CCG_CBL_DISC_DISABLE) */

#if (CCG_USB4_SUPPORT_ENABLE)
            /* Mark USB4 not active as Data Reset has been accepted. */
            app_get_status(port)->usb4_active = false;
#endif /* CCG_USB4_SUPPORT_ENABLE */            

            /* Switch to USB only configuration once data reset has been accepted. */
            if (dpm_stat->cur_port_type == PRT_TYPE_UFP)
            {
                set_mux (port, MUX_CONFIG_SS_ONLY, 0);
            }
            else
            {
                set_mux (port, MUX_CONFIG_ISOLATE, 0);
            }
            break;

        case APP_EVT_DATA_RESET_CPLT:
            /* DFP needs to re-enable USBx connections at completion of Data_Reset. */
            if (dpm_stat->cur_port_type == PRT_TYPE_DFP)
            {
                set_mux (port, MUX_CONFIG_SS_ONLY, 0);
            }
            break;

#if (CCG_USB4_SUPPORT_ENABLE)
        case APP_EVT_USB_ENTRY_CPLT:
            app_get_status(port)->usb4_data_rst_cnt = 0;
            app_get_status(port)->usb4_active = true;
            break;
#endif /* CCG_USB4_SUPPORT_ENABLE */

        case APP_EVT_PD_SINK_DEVICE_CONNECTED:
#if CCG_REV3_HANDLE_BAD_SINK
            gl_bad_sink_pd_status[port] = true;
#endif /* CCG_REV3_HANDLE_BAD_SINK */
            break;

        case APP_EVT_PKT_RCVD:
#if CHUNKING_NOT_SUPPORTED
            /* Make sure we stop the timer that initiates NOT_SUPPORTED response sending. */
            timer_stop(port, APP_CHUNKED_MSG_RESP_TIMER);
#endif /* CHUNKING_NOT_SUPPORTED */
            break;

        case APP_EVT_CBL_RESET_SENT:
            app_get_status(port)->cbl_rst_done = true;
            break;

#if (!CCG_SINK_ONLY)
        case APP_EVT_HR_PSRC_ENABLE:
#if BB_RETIMER_ENABLE
            set_retimer_status(port, true);
#endif /* BB_RETIMER_ENABLE */
            set_mux (port, MUX_CONFIG_SS_ONLY, 0);
            break;
#endif /* (!CCG_SINK_ONLY) */

#if ((ICL_ENABLE) && (PROCHOT_SUPP))
        case APP_EVT_TYPEC_RP_DETACH:
            /* Fall-through */
        case APP_EVT_PR_SWAP_ACCEPTED:
#if ((ICL_ENABLE) && (PROCHOT_SUPP))    
            ProcHot_Assert ();
#endif /* ((ICL_ENABLE) && (PROCHOT_SUPP)) */            
            break;
#endif /* ((ICL_ENABLE) && (PROCHOT_SUPP)) */

        default:
            /* Nothing to do. */
            break;
    }

    /* Pass the event notification to the fault handler module. */
    if (fault_event_handler (port, evt, dat))
    {
        skip_soln_cb = true;
    }

#if BATTERY_CHARGING_ENABLE
    bc_pd_event_handler (port, evt);
#endif /* BATTERY_CHARGING_ENABLE */

#if POWER_BANK
    pb_event_handler (port, evt);
#endif /* POWER_BANK */

    if (!skip_soln_cb)
    {
        /* Send notifications to the solution */
        sln_pd_event_handler(port, evt, dat);
    }
}

app_resp_t* app_get_resp_buf(uint8_t port)
{
    return &app_status[port].app_resp;
}

app_status_t* app_get_status(uint8_t port)
{
    return &app_status[port];
}

#if (defined (CCG5) || defined(CCG6) || defined(CCG5C) || defined(CCG6DF) || defined(CCG6SF))
bool app_is_port_enabled(uint8_t port)
{
    bool ret = true;

    if (port > NO_OF_TYPEC_PORTS)
        ret = false;

#if ((CCG_HPI_ENABLE) && (!CCG_LOAD_SHARING_ENABLE))
    /* Check if the port has been disabled through HPI. */
    if ((hpi_get_port_enable() & (1 << port)) == 0)
        ret = false;
#endif /* CCG_HPI_ENABLE */

    return ret;
}

/* Callback that will be called when there is any change to the V5V or VSYS supplies. */
void app_ccgx_supply_change_cb(uint8_t port, ccg_supply_t supply_id, bool present)
{
#if CCGX_V5V_CHANGE_DETECT
    const dpm_status_t *dpm_stat = dpm_get_info(port);

    /*
     * Currently we only handle V5V changes:
     * If V5V is removed, we exit active alternate modes if there is a cable which requires VConn.
     * If V5V is re-applied after being removed, we restart the alternate mode state machine.
     */
    if (supply_id == CCG_SUPPLY_V5V)
    {
        if (!present)
        {
            app_status[port].fault_status |= APP_PORT_V5V_SUPPLY_LOST;

#if (!CCG_VCONN_DISABLE)
            if (vconn_is_present (port))
            {
                vconn_change_handler (port, false);
            }
#endif /* (!CCG_VCONN_DISABLE) */
        }
        else
        {
            if ((app_get_status(port)->fault_status & APP_PORT_V5V_SUPPLY_LOST) != 0)
            {
                app_status[port].fault_status &= ~APP_PORT_V5V_SUPPLY_LOST;
                vconn_change_handler (port, true);
            }
        }
    }
#endif /* CCGX_V5V_CHANGE_DETECT */
}

#if (!CCG_BACKUP_FIRMWARE)
/* Callback used to receive fault notification from the HAL and to pass it on to the event handler. */
static void app_cc_sbu_fault_handler(uint8_t port, bool fault_type)
{
    app_event_handler(port, (fault_type ? APP_EVT_SBU_OVP : APP_EVT_CC_OVP), 0);
}
#endif /* (!CCG_BACKUP_FIRMWARE) */

#endif /* defined (CCG5) || defined(CCG6) || defined(CCG5C) || defined(CCG6DF) || defined(CCG6SF) */

void app_init(void)
{
    uint8_t port;

    /* For now, only the VDM handlers require an init call. */
    for (port = 0; port < NO_OF_TYPEC_PORTS; port++)
    {
        /* Initialize the VDM responses from the configuration table. */
        vdm_data_init(port);

        /* Set alt mode trigger based on config. */
        app_get_status(port)->alt_mode_trig_mask = get_pd_port_config(port)->alt_mode_trigger;

#if ((defined(CCG6)) && OTP_ENABLE)
        /* Init OT temperature tracking */
        app_otp_enable (port);
#endif /* ((defined(CCG6)) && OTP_ENABLE) */

#if (CCG_BB_ENABLE != 0)
        /*
         * Initialize the billboard interface. The billboard
         * interface shall not get initialized if it is not
         * enabled in configuration table.
         */
        bb_init(port);
#endif /* (CCG_BB_ENABLE != 0) */

#if BATTERY_CHARGING_ENABLE
        bc_init(port);
#endif /* BATTERY_CHARGING_ENABLE */

#if (ROLE_PREFERENCE_ENABLE)
        if (get_pd_port_config(port)->tbthost_cfg_tbl_offset != 0)
        {
#if (POWER_ROLE_PREFERENCE_ENABLE)
            app_pref_power_role[port] = (PD_GET_PTR_TBTHOST_CFG_TBL(port)->pref_pwr_role);
#endif /* (POWER_ROLE_PREFERENCE_ENABLE) */
            app_pref_data_role[port] = (PD_GET_PTR_TBTHOST_CFG_TBL(port)->pref_data_role);
        }
#endif /* (ROLE_PREFERENCE_ENABLE) */

#if ((DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP))
        /* Update alternate mode mask settings from config table. */
        {
            alt_cfg_settings_t *altcfg = PD_GET_PTR_ALT_MODE_TBL(port);

            app_status[port].dfp_alt_mode_mask = altcfg->dfp_mask;
            app_status[port].ufp_alt_mode_mask = altcfg->ufp_mask;
        }
#endif /* ((DFP_ALT_MODE_SUPP) || (UFP_ALT_MODE_SUPP)) */

        /* Update custom host config settings to the stack. */
        if (get_pd_port_config(port)->custom_host_tbl_offset != 0)
        {
            custom_host_cfg_settings_t *hostcfg = PD_GET_PTR_HOST_CFG_TBL(port);

            dpm_set_accessory_mode_disabled(port, hostcfg->acc_mode_disable);
            dpm_set_rp_detach_detect_disabled(port, hostcfg->rp_detach_disable);
        }
    }

#if PD_SRC_POWER_SHARE_ENABLE
    /* Excess power is not allocated to any ports. */
    app_src_extra_power_port = NO_OF_TYPEC_PORTS;
#endif /* PD_SRC_POWER_SHARE_ENABLE */

#if (CCG_TYPE_A_PORT_ENABLE == 1)
    /* For systems with TYPEA port. */
    if (get_pd_port_config(0)->type_a_enable)
    {
        type_a_port_enable();
    }
    else
    {
        /* Ensure that the state machine variables are initialized correctly. */
        type_a_port_disable();
    }
#endif /* (CCG_TYPE_A_PORT_ENABLE == 1) */

#if (defined (CCG5) || defined(CCG6) || defined(CCG5C) || defined(CCG6DF) || defined(CCG6SF))
#if (!CCG_BACKUP_FIRMWARE)
#if (CCG_CC_SBU_OVP_RETRY_LIMIT != 0)
    /* Set the retry limit where it is non-zero. */
    gl_app_fault_retry_limit[0][FAULT_TYPE_CC_OVP]  = CCG_CC_SBU_OVP_RETRY_LIMIT;
    gl_app_fault_retry_limit[0][FAULT_TYPE_SBU_OVP] = CCG_CC_SBU_OVP_RETRY_LIMIT;

#if CCG_PD_DUALPORT_ENABLE
    gl_app_fault_retry_limit[1][FAULT_TYPE_CC_OVP]  = CCG_CC_SBU_OVP_RETRY_LIMIT;
    gl_app_fault_retry_limit[1][FAULT_TYPE_SBU_OVP] = CCG_CC_SBU_OVP_RETRY_LIMIT;
#endif /* CCG_PD_DUALPORT_ENABLE */
#endif /* (CCG_CC_SBU_OVP_RETRY_LIMIT != 0) */

    /* Register a callback for notification of CC/SBU faults. */
    ccg_set_fault_cb(app_cc_sbu_fault_handler);
#endif /* (!CCG_BACKUP_FIRMWARE) */

#if ((!ICL_ENABLE) && (CCGX_V5V_CHANGE_DETECT))
    /* Register a handler that will be notified when there is any change in V5V or VSYS state. */
    pd_hal_set_supply_change_evt_cb(app_ccgx_supply_change_cb);
#endif /* ((!ICL_ENABLE) && (CCGX_V5V_CHANGE_DETECT)) */

#endif /* defined (CCG5) || defined(CCG6) || defined(CCG5C) || defined(CCG6DF) || defined(CCG6SF) */

#if APP_PPS_SINK_SUPPORT
    hpi_set_userdef_write_handler (hpi_user_reg_handler);
#endif /* APP_PPS_SINK_SUPPORT */
}

#if SYS_DEEPSLEEP_ENABLE
/* Implements CCG deep sleep functionality for power saving. */
bool system_sleep(void)
{
    uint8_t intr_state;
    bool dpm_slept = false;
    bool app_slept = false;
    bool retval = false;
#if BATTERY_CHARGING_ENABLE
    bool bc_slept = false;
#endif /* BATTERY_CHARGING_ENABLE */

    /* Do one DPM sleep capability check before locking interrupts out. */
    if (!dpm_is_idle ())
        return retval;

    intr_state = CyEnterCriticalSection();

    /*
     * We have to check the application layer, HPI and the Device Policy
     * Manager (DPM) to see if all of these modules are ready for sleep.
     * CCG can only enter deep sleep if all of these blocks are in an idle
     * state.
     *
     * Note: The respective sleep functions might be performing some
     * state updates as part of the idle check function; and therefore
     * the corresponding wakeup function needs to be called if they have
     * returned true to indicate that sleep is allowed.
     */
    if (app_sleep())
    {
        app_slept = true;

#if BATTERY_CHARGING_ENABLE
        if(bc_sleep() == true)
        {
            bc_slept = true;
#endif /* BATTERY_CHARGING_ENABLE */

            if (
#if CCG_HPI_ENABLE
                    (hpi_sleep_allowed()) &&
#endif /* CCG_HPI_ENABLE */

#if CCG_UCSI_ENABLE
                    (ucsi_sleep_allowed()) &&
#endif /* CCG_UCSI_ENABLE */

#if ICL_ENABLE
                    (icl_sleep_allowed()) &&
#endif /* ICL_ENABLE */

#if BB_RETIMER_ENABLE
                    (retimer_sleep_allowed()) &&
#endif /* BB_RETIMER_ENABLE */

#if CCG_SYNC_ENABLE
                    (ccg_sync_sleep_allowed()) &&
#endif /* CCG_SYNC_ENABLE */

#if (CCG_TEMP_BASED_VOLTAGE_THROTTLING || CCG_VIN_BASED_VOLTAGE_THROTTLING)
                    ccg_sensor_is_idle() &&
#endif /*(CCG_TEMP_BASED_VOLTAGE_THROTTLING || CCG_VIN_BASED_VOLTAGE_THROTTLING) */

#if CCG_LOAD_SHARING_ENABLE
                    ccg_ls_is_idle(0) &&
#endif /* CCG_LOAD_SHARING_ENABLE */
#if CCG_CABLE_COMP_ENABLE
                    ccg_cable_comp_is_idle(0) &&
#endif /* CCG_CABLE_COMP_ENABLE */
#if (CCG_TEMP_BASED_VOLTAGE_THROTTLING || CCG_VIN_BASED_VOLTAGE_THROTTLING || CCG_LOAD_SHARING_ENABLE)
                    ccg_power_throttle_sleep_allowed() &&
#endif /* (CCG_TEMP_BASED_VOLTAGE_THROTTLING || CCG_VIN_BASED_VOLTAGE_THROTTLING || CCG_LOAD_SHARING_ENABLE) */
                    (dpm_deepsleep())
               )
            {
                dpm_slept = true;

#if CCG_SYNC_ENABLE
                CCGComm_Sleep();
#endif /* CCG_SYNC_ENABLE */

                /*
                 * Check connection status of TYPE-A and TYPE-C ports to determine if deepsleep
                 * entry is allowed. If not,enter sleep mode to save power.
                 */
                if (
#if CCG_TYPE_A_PORT_ENABLE
                    (type_a_is_idle() == true) &&
#endif /* CCG_TYPE_A_PORT_ENABLE */
                    (app_deepsleep_allowed() == true)
                    )
                {
                    timer_enter_sleep();

                    /*
                     * CDT 224642: The I2C IDLE check needs to be done as the last step
                     * before device enters into sleep. Otherwise, the device may fail
                     * to wake up when there is an address match on the I2C interface.
                     */
#if ((CCG_HPI_ENABLE) || (RIDGE_SLAVE_ENABLE))
                    if (
#if CCG_HPI_ENABLE
                            (hpi_sleep())
#else
                            (1)
#endif /* CCG_HPI_ENABLE */
                            &&
#if RIDGE_SLAVE_ENABLE
                            (ridge_slave_sleep())
#else
                            (1)
#endif /* RIDGE_SLAVE_ENABLE */
                       )
#endif /* ((CCG_HPI_ENABLE) || (RIDGE_SLAVE_ENABLE)) */
                    {
#if (defined(CCG3PA) || defined(CCG3PA2))
                        pd_hal_set_reference(0, true);
#endif /* (defined(CCG3PA) || defined(CCG3PA2)) */

#if ((CCG_HPI_ENABLE) || (RIDGE_SLAVE_ENABLE))
                        if (
#if CCG_HPI_ENABLE
                                ((SCB_PRT[HPI_SCB_INDEX]->i2c_status & SCB_I2C_STATUS_BUS_BUSY) == 0)
#else
                                (1)
#endif /* CCG_HPI_ENABLE */
                                &&
#if (RIDGE_SLAVE_ENABLE)
                                ((SCB_PRT[RIDGE_SLAVE_SCB]->i2c_status & SCB_I2C_STATUS_BUS_BUSY) == 0)
#else
                                (1)
#endif /* RIDGE_SLAVE_ENABLE */
                           )
#endif /* ((CCG_HPI_ENABLE) || (RIDGE_SLAVE_ENABLE)) */
                        {
                            /* Device sleep entry. */
                            CySysPmDeepSleep();
                        }
#if (defined(CCG3PA) || defined(CCG3PA2))
                        pd_hal_set_reference(0, false);
#endif /* (defined(CCG3PA) || defined(CCG3PA2)) */
                        retval = true;
                    }
                }
                else
                {
#if (defined(CCG3PA) || defined(CCG3PA2) || defined(PAG1S))
                    /* Enter Sleep mode to save power. */
                    CySysPmSleep();
#endif /* (defined(CCG3PA) || defined(CCG3PA2) || defined(PAG1S)) */
                }
#if CCG_SYNC_ENABLE
                CCGComm_Wakeup();
#endif /* CCG_SYNC_ENABLE */
            }
#if BATTERY_CHARGING_ENABLE
        }
#endif /* BATTERY_CHARGING_ENABLE */
    }

    CyExitCriticalSection(intr_state);

    /* Call dpm_wakeup() if dpm_sleep() had returned true. */
    if(dpm_slept)
    {
        dpm_wakeup();
    }

    /* Call app_wakeup() if app_sleep() had returned true. */
    if(app_slept)
    {
        app_wakeup();
    }

#if BATTERY_CHARGING_ENABLE
    if(bc_slept)
    {
        bc_wakeup();
    }
#endif /* BATTERY_CHARGING_ENABLE */

    return retval;
}
#endif /* SYS_DEEPSLEEP_ENABLE */

#if VCONN_OCP_ENABLE
void app_vconn_ocp_cbk(uint8_t port, bool comp_out)
{
    /* Disable VConn since we hit a fault. */
    vconn_disable(port, dpm_get_info(port)->rev_pol);

    /* Notify application layer about fault. */
    app_event_handler(port, APP_EVT_VCONN_OCP_FAULT, NULL);
}
#endif /* VCONN_OCP_ENABLE */

bool vconn_enable(uint8_t port, uint8_t channel)
{
#if (!CCG_VCONN_DISABLE)
#if VCONN_OCP_ENABLE

    /* Do not attempt to enable VConn if fault was detected in present connection. */
    if ((app_get_status(port)->fault_status & APP_PORT_VCONN_FAULT_ACTIVE) != 0)
    {
        return false;
    }

#if CCG_VCONN_MON_WITH_ADC

    /* If Vconn current is being monitored through ADC, configure the associated IOs. */
    hsiom_set_config (APP_VCONN_MON_PORT_PIN_P1,APP_VCONN_MON_AMUX_INPUT_P1);
#if CCG_PD_DUALPORT_ENABLE
    hsiom_set_config (APP_VCONN_MON_PORT_PIN_P2,APP_VCONN_MON_AMUX_INPUT_P2);
#endif /* CCG_PD_DUALPORT_ENABLE */

    /*
     * 120us delay required as a settling time after HSIOM config to get a stable
     * ADC reading.
     */
    CyDelayUs(120);

#endif /* CCG_VCONN_MON_WITH_ADC */

    system_vconn_ocp_en(port, app_vconn_ocp_cbk);
#endif /* VCONN_OCP_ENABLE */

    /* Reset RX Protocol for cable */
    dpm_prot_reset_rx(port, SOP_PRIME);
    dpm_prot_reset_rx(port, SOP_DPRIME);

    if (pd_vconn_enable(port, channel) != CCG_STAT_SUCCESS)
    {
#if ((defined(CCG5)) || (defined(CCG5C)) || (defined(CCG6)) || (defined(CCG6DF)) || (defined(CCG6SF)))
        app_status[port].fault_status |= APP_PORT_V5V_SUPPLY_LOST;
#endif /* ((defined(CCG5)) || (defined(CCG5C)) || (defined(CCG6)) || (defined(CCG6DF)) || (defined(CCG6SF))) */
        return false;
    }
    return true;
#else
    return false;
#endif /* (!CCG_VCONN_DISABLE) */
}

void vconn_disable(uint8_t port, uint8_t channel)
{
#if (!CCG_VCONN_DISABLE)
    pd_vconn_disable(port, channel);

#if VCONN_OCP_ENABLE
    system_vconn_ocp_dis(port);
#endif /* VCONN_OCP_ENABLE */
#endif /* (!CCG_VCONN_DISABLE) */
}

bool vconn_is_present(uint8_t port)
{
#if (!CCG_VCONN_DISABLE)
    return pd_is_vconn_present( port, dpm_get_info(port)->rev_pol);
#else
    return false;
#endif /* (!CCG_VCONN_DISABLE) */
}

bool vbus_is_present(uint8_t port, uint16_t volt, int8 per)
{
    uint8_t level;
    uint8_t retVal;

#ifdef CCG4
    /*
     * OVP comparator is multiplexed with VBUS polling.
     * To avoid false output on OVP Trip pin when VBUS is polled
     * OVP trip pin is disconnected from OVP comp output and last
     * value of OVP comp output is driven via GPIO.
     */
    system_disconnect_ovp_trip(port);
#endif /* CCG4*/

#ifdef CCG3
    /* Ensure internal VBus divider is connected to AMUX where required. */
    pd_connect_vbus_div_to_amux(port);
#endif /* CCG3 */

    /*
     * Re-run calibration every time to ensure that VDDD or the measurement
     * does not break.
     */
    pd_adc_calibrate (port, APP_VBUS_POLL_ADC_ID);
    level = pd_get_vbus_adc_level(port, APP_VBUS_POLL_ADC_ID, volt, per);
    retVal = pd_adc_comparator_sample (port, APP_VBUS_POLL_ADC_ID, APP_VBUS_POLL_ADC_INPUT, level);

#ifdef CCG3
    /* Disconnect internal VBus divider from AMUX if possible. */
    pd_disconnect_vbus_div_from_amux(port);
#endif /* CCG3 */

#ifdef CCG4
    /* Connect OVP trip pin back to comparator output */
    system_connect_ovp_trip(port);
#endif /* CCG4*/

    return retVal;
}

uint16_t vbus_get_value(uint8_t port)
{
    uint16_t retVal;

#ifdef CCG4
    /*
     * OVP comparator is multiplexed with VBUS polling. To avoid false output
     * on OVP Trip pin when VBUS is polled OVP trip pin is disconnected from
     * OVP comp output and last value of OVP comp output is driven via GPIO.
     */
    system_disconnect_ovp_trip(port);
#endif /* CCG4 */

    /* Measure the actual VBUS voltage. */
    retVal = pd_hal_measure_vbus(port);

#ifdef CCG4
    /* Connect OVP trip pin back to comparator output */
    system_connect_ovp_trip(port);
#endif /* CCG4*/

    return retVal;
}

#if VCONN_OCP_ENABLE
bool system_vconn_ocp_en(uint8_t port, PD_ADC_CB_T cbk)
{
    if (cbk == NULL)
    {
        return false;
    }

#if (defined (CCG5) || defined(CCG6) || defined(CCG5C) || (defined(CCG6DF)) || (defined(CCG6SF)))

    /* Request the HAL to enable OCP detection with the appropriate debounce period. */
    pd_hal_vconn_ocp_enable(port, PD_GET_PTR_VCONN_OCP_TBL(port)->debounce, cbk);

#else /* !defined (CCG5) || defined(CCG6) || defined(CCG5C) || (defined(CCG6DF)) || (defined(CCG6SF)) */
#if CCG4_DOCK
    PPDSS_REGS_T gl_pdss[NO_OF_TYPEC_PORTS] =
    {
        PDSS0
#if CCG_PD_DUALPORT_ENABLE
            ,
        PDSS1
#endif /* CCG_PD_DUALPORT_ENABLE */
    };
    PPDSS_REGS_T pd = gl_pdss[port];
    pd->pfet300_ctrl |= PDSS_PFET300_CTRL_EN_COMP;
    pd->intr_1_cfg &= ~PDSS_INTR_1_CFG_V5V_CFG_MASK;
    pd->intr_1_cfg |= CYVAL_USBPD_V5V_CFG_POS_EDG_DIS_NEG_EDG_EN << PDSS_INTR_1_CFG_V5V_CFG_POS;
    pd->intr1_mask |= PDSS_INTR1_MASK_V5V_CHANGED_MASK;
#else /* !CCG4_DOCK */
    /* Configure ADC to detect VConn Over-Current condition. */
    pd_adc_comparator_ctrl(port, APP_VCONN_OCP_ADC_ID, APP_VCONN_OCP_ADC_INPUT,
            APP_VCONN_TRIGGER_LEVEL, PD_ADC_INT_RISING, cbk);
#endif /* CCG4_DOCK */
#endif /* defined (CCG5) || defined(CCG6) || defined(CCG5C) || (defined(CCG6DF)) || (defined(CCG6SF)) */
    return true;
}

void system_vconn_ocp_dis(uint8_t port)
{
#if (defined (CCG5) || defined(CCG6) || defined(CCG5C) || (defined(CCG6DF)) || (defined(CCG6SF)))

    /* Disable VConn OCP detection in the HAL. */
    pd_hal_vconn_ocp_disable(port);

#else /* !defined (CCG5) || defined(CCG6) || defined(CCG5C) || (defined(CCG6DF)) || (defined(CCG6SF)) */
#if CCG4_DOCK
    PPDSS_REGS_T gl_pdss[NO_OF_TYPEC_PORTS] =
    {
        PDSS0
#if CCG_PD_DUALPORT_ENABLE
            ,
        PDSS1
#endif /* CCG_PD_DUALPORT_ENABLE */
    };
    PPDSS_REGS_T pd = gl_pdss[port];
    pd->pfet300_ctrl &= ~PDSS_PFET300_CTRL_EN_COMP;
    pd->intr_1_cfg &= ~PDSS_INTR_1_CFG_V5V_CFG_MASK;
    pd->intr1_mask &= ~PDSS_INTR1_MASK_V5V_CHANGED_MASK;
#else /* !CCG4_DOCK */
    /* Disable the ADC used for VConn OCP detection. */
    pd_adc_comparator_ctrl(port, APP_VCONN_OCP_ADC_ID, 0, 0, 0, NULL);
#endif /* CCG4_DOCK */
#endif /* defined (CCG5) || defined(CCG6) || defined(CCG5C) || (defined(CCG6DF)) || (defined(CCG6SF)) */
}

#endif /* VCONN_OCP_ENABLE */

bool app_is_host_hpd_virtual(uint8_t port)
{
    bool ret = false;

#if RIDGE_I2C_HPD_ENABLE
    if (PD_GET_PTR_TBTHOST_CFG_TBL(port)->hpd_handling != 0)
    {
        ret = true;
    }
#endif /* RIDGE_I2C_HPD_ENABLE */

    return ret;
}

/* End of File */
