/**
 * @file bb_retimer.h
 *
 * @brief @{Burnside/Delta bridge retimer interface definitions.@}
 */

/*
 * Copyright (2014-2020), Cypress Semiconductor Corporation or a subsidiary of
 * Cypress Semiconductor Corporation. All rights reserved.
 *
 * This software, including source code, documentation and related materials
 * (“Software”), is owned by Cypress Semiconductor Corporation or one of its
 * subsidiaries (“Cypress”) and is protected by and subject to worldwide patent
 * protection (United States and foreign), United States copyright laws and
 * international treaty provisions. Therefore, you may use this Software only
 * as provided in the license agreement accompanying the software package from
 * which you obtained this Software (“EULA”).
 *
 * If no EULA applies, Cypress hereby grants you a personal, nonexclusive,
 * non-transferable license to copy, modify, and compile the Software source
 * code solely for use in connection with Cypress’s integrated circuit
 * products. Any reproduction, modification, translation, compilation, or
 * representation of this Software except as specified above is prohibited
 * without the express written permission of Cypress. Disclaimer: THIS SOFTWARE
 * IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress reserves the
 * right to make changes to the Software without notice. Cypress does not
 * assume any liability arising out of the application or use of the Software
 * or any product or circuit described in the Software. Cypress does not
 * authorize its products for use in any products where a malfunction or
 * failure of the Cypress product may reasonably be expected to result in
 * significant property damage, injury or death (“High Risk Product”). By
 * including Cypress’s product in a High Risk Product, the manufacturer of such
 * system or application assumes all risk of such use and in doing so agrees to
 * indemnify Cypress against all liability.
 */

#ifndef _BB_RETIMER_H_
#define _BB_RETIMER_H_

/*******************************************************************************
 * Header files including
 ******************************************************************************/

#include <pd.h>
    
/*******************************************************************************
 * Enumerated Types
 ******************************************************************************/

/**
 * @typedef rt_evt_t
 * @brief List of retimer related events to be handled by the CCGx firmware.
 */
typedef enum {
    RT_EVT_VSYS_REMOVED,                /**< VSYS supply to CCGx device removed. Need to disable retimer. */
    RT_EVT_VSYS_ADDED,                  /**< VSYS supply to CCGx device restored. */
    RT_EVT_RETRY_STATUS,                /**< Retimer status update has to be retried. */
    RT_EVT_UPDATE_STATUS,               /**< Retimer status has to be changed. */
    RT_EVT_FORCE_PWR_CHNG,              /**< Force retimer power ON requested. */
    RT_EVT_WRITE_DEBUG_MODE,            /**< Write to retimer debug register requested. */
    RT_EVT_READ_DEBUG_MODE,             /**< Read from retimer debug register requested. */
    RT_MAX_EVTS                         /**< Invalid event. */
} rt_evt_t;

/*******************************************************************************
 * Helper Macros
 ******************************************************************************/

#define BB_DEBUG_MODE_SIZE              (4u)
/**< Size of the BB Debug Mode register. */

#define BB_DEBUGMODE_POLL_COUNT         (50u)
/**< Number of times to poll the retimer before giving up. */

#define MAX_NO_OF_RETIMERS              (2u)
/**< Max number of retimers available on any platform. */

#define RETIMER_CFG_AVAILABLE           (0x80)
/**< Retimer Configuration bit: is retimer available?. */

#define RETIMER_CFG_SLAVE_ADDR          (0x7F)
/**< Retimer Configuration bit: 7-bit slave address. */

#define BB_CONN_STATE_REG               (0x04)
/**< BB Register: Connection State. */

#define BB_STATUS_REG                   (0x05)
/**< BB Register: TBT Status. */

#define BB_DEBUG_MODE_REG               (0x07)
/**< BB Register: Debug Mode. */

#if ICL_ENABLE

#define BB_STATUS_MASK                  (0xBEFFDFFFu)
/**< TBT Status bits that are used in BB. */

#else

#define BB_STATUS_MASK                  (0xBEFFDFFFu)
/**< TBT Status bits that are used in BB. */

#endif /* ICL_ENABLE. */    

#define BB_USB_3_SPEED_MASK             (0x00000040u)
/**< Bit 6 USB3 Speed Mask. */

#define BB_IRQ_HPD_MASK                 (0x00004000u)
/**< Bit 14 IRQ HPD Mask. */

#define BB_STATUS_Sx_ACTIVE             (1 << 24)
/**< Bit 30 (previously reserved) indicates S0=0 vs Sx=1. */

#define BB_DEBUG_MODE_RX_LOCKED         (0x8000u)
/**< 'Rx Locked' bit in the status register. */

#define BB_DEBUG_MODE_USB_COMPLIANCE    (0xD0000000u)
/**< Bits to be set in DEBUG_MODE for USB Compliance mode entry
     Bit 31 (Global Enable) = 1
     Bit 30 (Compliance Enabled) = 1
     Bit 29 (USB Compliance Mode) = 1
     Bit 28 (HTI Mode) = 0 */

#define BB_DEBUG_MODE_CLEAR             (0u)
/**< BB Register : Clear. */

#define BB_DEBUG_MODE_DEFAULT           (0xFFFFFFFFu)
/**< Default mask used to "AND" all debug mode data from retimers. */
    
#define BB_DBR_DEBUG_POLL_DELAY         (100u)
/**< Add a 100ms timer to retry reading the DEBUG register from the retimer. */

#define BB_DBR_DEBUG_MODE_REG_WRITE_DELAY (85u)
/**< Add a 85ms timer to delay the write to the debug mode register as part of
 * the retimer enable sequence. */
    
#define BB_DBR_WAKEUP_DELAY             (40u)
/**< Retimer was in low power and NAKed the request. The initial I2C request will wake the retimer up after which
  * it starts re-loading firmware. Give it ~40ms to do this and start accepting state change requests.
  * (Intel retimers unload their firmare upon entering low power mode)
  * Retry status update after 40ms. 
  * This is as per spec 614432-tgl-pd-ctrl-intfreq-burnsidebridgeretimer-prd-rev0p95 (sec 7.2)*/

#define RETIMER_I2C_TIMEOUT             (0x64)
/**< Timeout for the function completion is 100 ms. */

/*******************************************************************************
 * Extern variable definitions
 ******************************************************************************/

/** @cond DOXYGEN_HIDE */

extern uint8_t gl_system_state;
extern bool    is_retimer_configured;

/** @endcond */

/*******************************************************************************
 * Function Definitions
 ******************************************************************************/

/**
 * @brief This initializes the retimer I2C and GPIO interfaces
 * @return None
 */
void retimer_init(void);

/**
 * @brief Function check whether retimers are present on the corresponding Type-C port.
 * @param port Type-C port index.
 * @return True if there is at least one retimer associated with the port.
 */
bool retimer_is_present(uint8_t port);

/**
 * @brief Enable retimer on the specific port. This releases its reset
 * and turns the power gate on.
 *
 * @param port Port index the function is performed for.
 * @param delay_enable Flag that indicates that enable sequence should be delayed.
 *
 * @return None
 */
void retimer_enable(uint8_t port, bool delay_enable);

/**
 * @brief Disable retimer on the specific port. This holds its reset
 * and turns the power gate off.
 *
 * @param port Port index the function is performed for.
 * @param delay_enable Flag that indicates that disable sequence should be delayed.
 *
 * @return None
 */
void retimer_disable(uint8_t port, bool delay_enable);

/**
 * @brief Set the status of retimer. 
 * @param port PD port associated with the retimer to be updated.
 * @param status Enable or disable flag for retimer.
 * @return None
 */
void set_retimer_status(uint8_t port, uint8_t status);

/**
 * @brief Set an event on the retimer module
 *
 * @param port Port index the function is performed for.
 * @param evt Event to signal on the module
 *
 * @return None
 */
void retimer_set_evt(uint8_t port, rt_evt_t evt);

/**
 * @brief Clear an event on the retimer module
 *
 * @param port Port index the function is performed for.
 * @param evt Event to clear on the module
 *
 * @return None
 */
void retimer_clr_evt(uint8_t port, rt_evt_t evt);

/**
 * @brief Start polling the debug mode register on the retimer
 *
 * @param  port Port index the function is performed for.
 * @param  mode_data Debug mode data
 * @param  poll_count Number of polling attempts
 * @param  delay_enable Delay the debug mode register write
 *
 * @return None
 */
void retimer_start_debug_poll(uint8_t port, uint32_t mode_data, uint8_t poll_count, uint8_t delay_enable);

/**
 * @brief Update the retimer status
 *
 * @param  port Port index the function is performed for.
 * @param  status Retimer status
 * @param  force_update Force update of retimer status
 *
 * @return bool Result of the update
 */
bool retimer_status_update(uint8_t port, uint32_t status, bool force_update);

/**
 * @brief Check if retimer update is pending
 *
 * @param  port Port index the function is performed for.
 * @return bool Update pending status
 */
bool retimer_update_is_pending(uint8_t port);

/**
 * @brief Run the retimer task for that port
 *
 * @param port Port index the retimer functions are handled for
 *
 * @return None
 */
void retimer_task(uint8_t port);

/**
 * @brief Set the retimer slave address
 * @return None
 */
void retimer_set_slave_address(void);

/**
 * @brief Check if retimer interface can go to sleep
 * @return bool Can enter sleep or not
 */
uint8_t retimer_sleep_allowed(void);

/**
 * @brief Indicator of whether host is in compliance mode
 *
 * @param compl_mode Mode status
 *
 * @return None
 */
void retimer_set_compl_mode(uint8_t compl_mode);

/**
 * @brief Enable retimer on the specific port. 
 *
 * @param port Port index the function is performed for.
 *
 * @return None
 */
void retimer_force_enable(uint8_t port);

/**
 * @brief  Write to the retimer register. 
 *
 * @param port Port index the function is performed for.
 *
 * @param reg Register the function is performed for.
 *
 * @param value Register value for update.
 *
 * @return None
 */
bool retimer_reg_write(uint8_t port, uint8_t reg, uint32_t value);

/**
 * @brief Wrapper for performing writes using the RTM PSoC Creator Component.
 * @param slave_address Address of the slave
 * @param offset Offset in the slave to write to
 * @param data Pointer to the buffer containing data to be written to the slave
 * @param num_bytes Number of bytes to write
 * @return True if write was successful otherwise False
 */
bool retimer_write_wrapper(uint8_t slave_address, uint8_t offset, uint8_t* data, uint8_t num_bytes);

/**
 * @brief Wrapper for performing reads using the RTM PSoC Creator Component.
 * @param slave_address Address of the slave
 * @param offset Offset in the slave to read from
 * @param data_buffer Pointer to the buffer into which data is read
 * @param num_bytes Number of bytes to be read
 * @return True if read was successful otherwise False
 */
bool retimer_read_wrapper(uint8_t slave_address, uint8_t offset, uint8_t* data_buffer, uint8_t num_bytes);

#endif /* _BB_RETIMER_H_ */

/* End of File */

