/**
 * @file icl.h
 *
 * @brief @{Ice Lake specific logic.@}
 */

/*
 * Copyright (2014-2020), Cypress Semiconductor Corporation or a subsidiary of
 * Cypress Semiconductor Corporation. All rights reserved.
 *
 * This software, including source code, documentation and related materials
 * (“Software”), is owned by Cypress Semiconductor Corporation or one of its
 * subsidiaries (“Cypress”) and is protected by and subject to worldwide patent
 * protection (United States and foreign), United States copyright laws and
 * international treaty provisions. Therefore, you may use this Software only
 * as provided in the license agreement accompanying the software package from
 * which you obtained this Software (“EULA”).
 *
 * If no EULA applies, Cypress hereby grants you a personal, nonexclusive,
 * non-transferable license to copy, modify, and compile the Software source
 * code solely for use in connection with Cypress’s integrated circuit
 * products. Any reproduction, modification, translation, compilation, or
 * representation of this Software except as specified above is prohibited
 * without the express written permission of Cypress. Disclaimer: THIS SOFTWARE
 * IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress reserves the
 * right to make changes to the Software without notice. Cypress does not
 * assume any liability arising out of the application or use of the Software
 * or any product or circuit described in the Software. Cypress does not
 * authorize its products for use in any products where a malfunction or
 * failure of the Cypress product may reasonably be expected to result in
 * significant property damage, injury or death (“High Risk Product”). By
 * including Cypress’s product in a High Risk Product, the manufacturer of such
 * system or application assumes all risk of such use and in doing so agrees to
 * indemnify Cypress against all liability.
 */

#ifndef _ICL_H_
#define _ICL_H_

/*******************************************************************************
 * Header files including
 ******************************************************************************/
#include <stdint.h>
#include <pd.h>
#include <hal_ccgx.h>

/*******************************************************************************
 * MACRO Definition
 ******************************************************************************/

/**
 * @typedef icl_reg_t
 * @brief List of HPI control registers supported for IceLake platform management.
 */
typedef enum {
    ICL_CTRL_REG              = 0x0040,         /**< ICL Control Register*/
    ICL_STS_REG               = 0x0042,         /**< ICL Status Register */
    ICL_DBG_REG               = 0x0044,         /**< ICL Debug Register */
    ICL_BB_RETIMER_FORCE_PWR  = 0x0045,         /**< Register to force power to retimer. */
    ICL_BB_RETIMER_CMD_REG    = 0x0046,         /**< ICL HPI Retimer Command register 45:47h */ 
    ICL_BB_RETIMER_DAT_REG    = 0x0048,         /**< ICL HPI Retimer Data Register 48:4Bh */
} icl_reg_t;
    
/**
 * @typedef icl_evt_t
 * @brief List of IceLake platform specific events to be handled by the CCG firmware.
 */
typedef enum {
    ICL_EVT_DEQUEUE_HPD,                        /**< 0x00 : ICL Dequeue HPD event */
    ICL_EVT_CHANGE_MUX_STATE,                   /**< 0x01 : ICL MUX change state event */
    ICL_MAX_EVTS                                /**< 0x02 : ICL Maximum events */
} icl_evt_t;

#define ICL_CTRL_CMD_FORCE_TBT_MODE         (0x01u)
/**< HPI command code to trigger retimer firmware update mode. */

#define ICL_CTRL_CMD_SOC_TO_EVT_DISABLE     (0x80u)
/**< HPI register bit to disable SoC Ack Timeout event. */

#define ICL_STS_REG_FORCE_TBT_MODE          (1 << 0)
/**< HPI status register bit that indicates retimer has been forced into USB4/TBT mode. */
    
#define ICL_RFU_EXIT_DURATION               (12u)
/**< Delay in ms before Retimer is brought out of firmware update mode. */

#define ICL_ADP_DETACH_DEBOUNCE_TIME       (100)
/**< Debounce time in ms used to confirm that DC barrel adapter has been removed. */

#define ICL_ADP_ATTACH_DEBOUNCE_TIME       (5)
/**< Debounce time in ms used to confirm that DC barrel adapter has been connected. */

#define ADP_STATE_INIT                     (2)
/**< Initial state of the DC barrel adapter detect input. Setting to a value other than 0 and 1. */

#define ADP_NEGEDGE                        (0)  /**< Negative edge seen on DC barrel presence detect pin. */
#define ADP_POSEDGE                        (1)  /**< Positive edge seen on DC barrel presence detect pin. */
#define ADP_EDGE_NONE                      (2)  /**< No transition seen on DC barrel presence detect pin. */

/*******************************************************************************
 * Function Declaration
 ******************************************************************************/

/**
 * @brief Initialize logic needed for Ice Lake designs
 *
 * @return None
 */
void icl_init(void);

/**
 * @brief Run ICL RVP-specific tasks
 * @param port Port index
 * @return None
 */
void icl_task(uint8_t port);

#if ICL_OCP_ENABLE

/**
 * @brief Register a function to be called when OCP happens. Note that fault should be indicated on pins
 * mapped to the same GPIO port for all PD ports.
 *
 * @param port Port index
 * @param enable Enable or disable OCP
 * @param oc_pin Pin on which OCP fault is indicated.
 * @param cbk Callback to be called
 *
 * @return None
 */
void icl_ocp_cfg(uint8_t port, bool enable, uint8_t oc_pin, vbus_ocp_cbk_t cbk);

#endif /* ICL_OCP_ENABLE */

/**
 * @brief Set an event on the ICL module
 * @param port Port index
 * @param evt ICL event
 * @return None
 */
void icl_set_evt(uint8_t port, icl_evt_t evt);

/**
 * @brief Helper function to get VSYS state
 * @return bool True if VSYS is available else False
 */
bool icl_vsys_is_present(void);

/**
 * @brief Does ICL module allow CCG to enter deep-sleep
 * @return bool
 */
bool icl_sleep_allowed(void);

#endif /* _ICL_H_ */

/* End of File */
