/**
 * @file intel_ridge_internal.h
 *
 * @brief @{Intel Alpine/Titan Ridge register interface header file.@}
 */

/*
 * Copyright (2014-2020), Cypress Semiconductor Corporation or a subsidiary of
 * Cypress Semiconductor Corporation. All rights reserved.
 *
 * This software, including source code, documentation and related materials
 * (“Software”), is owned by Cypress Semiconductor Corporation or one of its
 * subsidiaries (“Cypress”) and is protected by and subject to worldwide patent
 * protection (United States and foreign), United States copyright laws and
 * international treaty provisions. Therefore, you may use this Software only
 * as provided in the license agreement accompanying the software package from
 * which you obtained this Software (“EULA”).
 *
 * If no EULA applies, Cypress hereby grants you a personal, nonexclusive,
 * non-transferable license to copy, modify, and compile the Software source
 * code solely for use in connection with Cypress’s integrated circuit
 * products. Any reproduction, modification, translation, compilation, or
 * representation of this Software except as specified above is prohibited
 * without the express written permission of Cypress. Disclaimer: THIS SOFTWARE
 * IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress reserves the
 * right to make changes to the Software without notice. Cypress does not
 * assume any liability arising out of the application or use of the Software
 * or any product or circuit described in the Software. Cypress does not
 * authorize its products for use in any products where a malfunction or
 * failure of the Cypress product may reasonably be expected to result in
 * significant property damage, injury or death (“High Risk Product”). By
 * including Cypress’s product in a High Risk Product, the manufacturer of such
 * system or application assumes all risk of such use and in doing so agrees to
 * indemnify Cypress against all liability.
 */

#ifndef _INTEL_RIDGE_INTERNAL_H_
#define _INTEL_RIDGE_INTERNAL_H_

/*****************************************************************************
 * Header files including
 *****************************************************************************/

#include <alt_mode_hw.h>
#include <status.h>
#include <hpd.h>
#include <config.h>    

#define RIDGE_DISCON_STATE_MASK                  (0x00)
/** @brief Disconnect state bit mask for Status register. */

#define RIDGE_DATA_CONN_PRESENT                  (0x01)
/** @brief Data present field in the status register. */

#if CCG_USB4_SUPPORT_ENABLE
#define RIDGE_DATA_CONN_MASK                     (0x3EC1D17D)
/** @brief Data connection bit mask for Status register. */    
#else    
#define RIDGE_DATA_CONN_MASK                     (0x1D179)
/** @brief Data connection bit mask for Status register. */
#endif /* CCG_USB4_SUPPORT_ENABLE */    

#define RIDGE_USB4_CONN_MASK                     (0x800011)
/** @brief USB4 connection bit mask for Status register. */

#define RIDGE_USB_STATE_MASK                     (0x31)
/** @brief USB only state bit mask for  Status register. */

#define RIDGE_SAFE_STATE_MASK                    (0x01)
/** @brief Safe state bit mask for  Status register. */

#define RIDGE_DP_4_LANE_MASK                     (0x111)
/** @brief 4 lane DP alt mode state bit mask for  Status register. */

#define RIDGE_DP_2_LANE_MASK                     (0x531)
/** @brief 2 lane DP alt mode state bit mask for Status register. */

#define DP_PIN_CONFIG_C                          (4u)
/** @brief Pin assigment mask for C pin assigment. */

#define DP_PIN_CONFIG_F                          (0x20u)
/** @brief Pin assigment mask for F pin assigment. */
#if ICL_ENABLE
#define RIDGE_TBT_MODE_MASK                      (0x10011)    
#else    
#define RIDGE_TBT_MODE_MASK                      (0x10001)
/** @brief TBT alt mode state bit mask for Titan register. */
#endif /* ICL_ENABLE */    

#define RIDGE_DEBUG_MODE_MASK                    (0x11)
/** @brief Debug alt mode state bit mask for Status register. */

#define TBT_HOST_CONN_MASK                       (0x01)
/** @brief TBT HOST Connected bit mask for Command register. */

#define TR_IRQ_ACK_MASK                          (0x2000)
/** @brief HPD IRQ ACK bit mask for Titan Ridge command register. */

#define TR_HPD_IRQ_MASK                          (0x4000)
/** @brief HPD IRQ bit mask for Titan Ridge command/status register. */

#define TR_HPD_LVL_MASK                          (0x8000)
/** @brief HPD Level bit mask for Titan Ridge command register. */

#define TR_USB_HOST_CONN_MASK                    (0x10)
/** @brief USB HOST Connected bit mask for Titan Ridge command register. */

#define TR_DP_HOST_CONN_MASK                     (0x20)
/** @brief DP HOST Connected bit mask for Titan Ridge command register. */

#define TR_ACT_CBL_MASK                          (0x400000)
/** @brief Active cable bit mask for Titan Ridge command register. */

#define TR_RETIMER_MASK                          (0x4)
/** @brief Retimer bit mask for Titan Ridge command register. */

#define MR_IN_SAFE_STATE_MASK                    (0x4)
/** @brief Safe State bit mask for Maple Ridge command register. */

#define MR_DATA_RST_REQ_MASK                     (0x1000)
/** @brief Data Reset Request Connected bit mask for Maple Ridge command register. */

#define MR_PWR_RST_REQ_MASK                      (0x2000)
/** @brief Power Reset Request Connected bit mask for Maple Ridge command register. */

#define MR_READY_MASK                            (0x4000)
/** @brief Ridge is ready to be functional bit mask for Maple Ridge command register. */

#define RIDGE_STATUS_OCP_MASK                    (0x08)
/** @brief Over-Current status bit in the status register. */

#define EUDO_HOST_PRES_OFFSET                    (13u)
/** @brief Offset of Host present field in USB4 Enter VDO. */

/*****************************************************************************
 * Data Struct Definition
 *****************************************************************************/

/**
  @union ridge_reg_t
  @brief Union to hold AR/TR Registers
 */
typedef union
{
    uint32_t val;                       /**< Integer field used for direct manipulation of reason code. */

    /** @brief Struct containing USB-PD controller status to be reported to Alpine/Titan Ridge.
        Using the structure definition corresponding to Titan Ridge in all cases.
     */
    struct USBPD_STATUS_REG
    {
        uint32_t data_conn_pres  : 1;   /**< B0: Whether data connection is present. */
        uint32_t conn_orien      : 1;   /**< B1: CC polarity. */

#if CCG_USB4_SUPPORT_ENABLE        
        uint32_t retimer         : 1;   /**< B2: Retimer or Redriver. */    
#else  
        uint32_t active_cbl      : 1;   /**< B2: Active cable. From B22 of Cable MODE Response. */    
#endif /* CCG_USB4_SUPPORT_ENABLE */  

        uint32_t ovc_indn        : 1;   /**< B3: Over-Current indication. */   
        uint32_t usb2_conn       : 1;   /**< B4: USB 2.0 connection. Set when no ALT MODES are active. */
        uint32_t usb3_conn       : 1;   /**< B5: USB 3.1 connection. Set when no ALT MODES are active. */
        uint32_t usb3_speed      : 1;   /**< B6: USB Gen2/Gen1 speed. From B18-B16 of Cable MODE Response. */
        uint32_t usb_dr          : 1;   /**< B7: Data role. DFP=0, UFP=1. */
        uint32_t dp_conn         : 1;   /**< B8: DP connection status. */
        uint32_t dp_role         : 1;   /**< B9: DP direction. Source=0, Sink=1. */
        uint32_t dp_pin_assign   : 2;   /**< B[11-10]: DP pin assignment. 4-lane='b00 2-lane='b01 */
        uint32_t dbg_acc_mode    : 1;   /**< B12: USB Type-C Debug accessory mode. */

#if CCG_USB4_SUPPORT_ENABLE        
        uint32_t rsvd1           : 1;   /**< B13: Reserved. */
#else   
        uint32_t irq_ack         : 1;   /**< B13: Set after receiving GoodCRC from Attention message with IRQ_HPD. */
#endif /* CCG_USB4_SUPPORT_ENABLE */  

        uint32_t hpd_irq         : 1;   /**< B14: HPD IRQ received from DP Sink. */
        uint32_t hpd_lvl         : 1;   /**< B15: HPD level received from DP Sink. */
        uint32_t tbt_conn        : 1;   /**< B16: TBT connection status. */
        uint32_t tbt_type        : 1;   /**< B17: TBT type. From B16 of UFP MODE Response. */
        uint32_t cbl_type        : 1;   /**< B18: TBT cable type. From B21 of Cable MODE Response. */
        uint32_t pro_dock_detect : 1;   /**< B19: Reporting of vPro Support. */
        uint32_t act_link_train  : 1;   /**< B20: Active TBT link training. From B23 of Cable MODE Response. */
        uint32_t dbg_alt_m_conn  : 1;   /**< B21: NIDnT Alt mode defined in MIPI SVID = 0xFF03. */

#if CCG_USB4_SUPPORT_ENABLE     
        uint32_t active_cbl      : 1;   /**< B22: Active cable or passive cable. */    
        uint32_t usb4_conn       : 1;   /**< B23: USB4 connection status. */
#else   
        uint32_t rsvd2           : 1;   /**< B22: Reserved. */
        uint32_t force_lsx       : 1;   /**< B23: Set to zero. */
#endif /* CCG_USB4_SUPPORT_ENABLE */          

        uint32_t pwr             : 1;   /**< B24: Indicates PWR mismatch (Used only in TBT BPD). */
        uint32_t cbl_spd         : 3;   /**< B[27-25]: Cable speed. From B18-B16 of Cable MODE Response. */
        uint32_t cbl_gen         : 2;   /**< B[29-28]: Cable generation. From B20-19 of Cable MODE Response. */
        uint32_t rsvd4           : 1;   /**< B30: Reserved. */
        uint32_t interrupt_ack   : 1;   /**< B31: Interrupt indication (Set by EC when in I2C slave Mode). */
    }ridge_stat;                        /**< PD-controller status. */

    /** @brief Struct containing USB-PD controller command register fields for Alpine/Titan Ridge. */
    struct USBPD_CMD_REG
    {
        uint32_t tbt_host_conn   : 1;   /**< B0: TBT host connected. */
#if CCG_USB4_SUPPORT_ENABLE
        uint32_t rsvd0           : 1;   /**< B1: Reserved. */
#else
        uint32_t soft_rst        : 1;   /**< B1: Issue USB-PD Controller soft reset. */
#endif /* CCG_USB4_SUPPORT_ENABLE */
        uint32_t i2c_int_ack     : 1;   /**< B2: Alpine/Titan Ridge acknowledge for the interrupt. */
#if CCG_USB4_SUPPORT_ENABLE
        uint32_t in_safe_state   : 1;   /**< B3: Ridge DP sets this after confirming high speed I/Os are in USB Type-C Safe State.. */
#else
        uint32_t rsvd1           : 1;   /**< B3: Reserved. */
#endif /* CCG_USB4_SUPPORT_ENABLE */
        uint32_t usb_host_conn   : 1;   /**< B4: Indicates that USB Host is connected (TBT device only). */
        uint32_t dp_host_conn    : 1;   /**< B5: Indicates that DP Host is connected (TBT device only). */
#if BB_RETIMER_ENABLE
        uint32_t rsvd3           : 6;   /**< B[11-6]: Reserved. */
        uint32_t wr_to_retimer   : 1;   /**< B12: If set, write following 4 bytes to retimer's Debug Mode */
#else        
        uint32_t rsvd3           : 7;   /**< B[12-6]: Reserved. */
#endif 
        uint32_t irq_ack         : 1;   /**< B13: IRQ ACK PD Controller to Titan Ridge. */
        uint32_t hpd_irq         : 1;   /**< B14: HPD IRQ when acting as DP Sink. */
        uint32_t hpd_lvl         : 1;   /**< B15: HPD level when acting as DP Sink. */
#if CCG_USB4_SUPPORT_ENABLE
        uint32_t data_rst_req    : 1;   /**< B16: Indicates to the PD that Data Reset should be done. */
        uint32_t pwr_rst_req     : 1;   /**< B17: Indicates to the PD that Power Reset should be done. */
        uint32_t ridge_ready     : 1;   /**< B18: Indicates to the PD that the Ridge is ready to be functional. */
        uint32_t rsvd4           : 13;  /**< B[31-19]: Reserved. */
#else
        uint32_t rsvd4           : 16;  /**< B[31-16]: Reserved. */
#endif /* CCG_USB4_SUPPORT_ENABLE */

    }usbpd_cmd_reg;                     /**< PD-controller command register. */

}ridge_reg_t;

/**
 * @brief Type of callback function used for notification of control register changes.
 */
typedef void (*ridge_ctrl_change_cb_t)(
        uint8_t port                    /**< Port on which control register change happened. */
        );

/*****************************************************************************
 * Global Function Declaration
 *****************************************************************************/

/**
 * @brief Register a callback for notification of data control register changes.
 *
 * @param port PD port index.
 * @param cb Pointer to callback function.
 * @return None
 */
void ridge_set_ctrl_change_cb(uint8_t port, ridge_ctrl_change_cb_t cb);

#endif /*_INTEL_RIDGE_INTERNAL_H_ */

/* [] END OF FILE */

