/**
 * @file ridge_slave.h
 *
 * @brief @{Alpine/Titan Ridge I2C slave interface header file.@}
 */

/*
 * Copyright (2014-2020), Cypress Semiconductor Corporation or a subsidiary of
 * Cypress Semiconductor Corporation. All rights reserved.
 *
 * This software, including source code, documentation and related materials
 * (“Software”), is owned by Cypress Semiconductor Corporation or one of its
 * subsidiaries (“Cypress”) and is protected by and subject to worldwide patent
 * protection (United States and foreign), United States copyright laws and
 * international treaty provisions. Therefore, you may use this Software only
 * as provided in the license agreement accompanying the software package from
 * which you obtained this Software (“EULA”).
 *
 * If no EULA applies, Cypress hereby grants you a personal, nonexclusive,
 * non-transferable license to copy, modify, and compile the Software source
 * code solely for use in connection with Cypress’s integrated circuit
 * products. Any reproduction, modification, translation, compilation, or
 * representation of this Software except as specified above is prohibited
 * without the express written permission of Cypress. Disclaimer: THIS SOFTWAR/TRE
 * IS PROVIDED AS-IS, WITH NO WAR/TRRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED WAR/TRRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PAR/TRTICULAR/TR PURPOSE. Cypress reserves the
 * right to make changes to the Software without notice. Cypress does not
 * assume any liability arising out of the application or use of the Software
 * or any product or circuit described in the Software. Cypress does not
 * authorize its products for use in any products where a malfunction or
 * failure of the Cypress product may reasonably be expected to result in
 * significant property damage, injury or death (“High Risk Product”). By
 * including Cypress’s product in a High Risk Product, the manufacturer of such
 * system or application assumes all risk of such use and in doing so agrees to
 * indemnify Cypress against all liability.
 */

#ifndef _RIDGE_SLAVE_H_
#define _RIDGE_SLAVE_H_

#include <stdbool.h>
#include <stdint.h>
#include "config.h"
#include "status.h"
#include "i2c.h"

#define RIDGE_SLAVE_SCB_INDEX                      (0x1)
/**< Default SCB index used for the Ridge/SoC Slave interface. */

#define RIDGE_SLAVE_SCB_CLOCK_FREQ                 (I2C_SCB_CLOCK_FREQ_1_MHZ)
/**< Maximum I2C clock frequency used on the Ridge/SoC slave interface. */

#define RIDGE_SLAVE_MIN_WRITE_SIZE                 (2)
/**< Minimum slave write size: Corresponds to slave address + register address. */

#define RIDGE_SLAVE_MAX_WRITE_SIZE                 (16)
/**< Maximum slave write size: We should never get writes longer than 16 bytes. */

#define RIDGE_SLAVE_MAX_READ_SIZE                  (16)
/**< Maximum slave read size. */
    
#if ICL_SLAVE_ENABLE

#define PMC_SLAVE_ADDR_PORT0                            (0x50)
/**< Port 0 slave default slave address. */

#define PMC_SLAVE_ADDR_PORT1                            (0x51)
/**< Port 1 slave default slave address. */
    
#define ICL_SOC_ACK_TIMEOUT_PERIOD                      (500u)
/**< t_SOCAckTimeout period defined in PD controller specs from Intel. */

#define RIDGE_SLAVE_ADDR_MASK_GENERAL                   (0x80)
/**< @brief I2C slave address mask to be applied on the incoming slave address. */

#else    

#define RIDGE_SLAVE_ADDR_P0                        (0x38)
/**< Slave address associated with USB-PD port number 0. This is defined by Intel. */

#define RIDGE_SLAVE_ADDR_P1                        (0x3F)
/**< Slave address associated with USB-PD port number 1. This is defined by Intel. */

#define RIDGE_SLAVE_ADDR_MASK                      (0xF0)
/**< @brief I2C slave address mask to be applied on the incoming slave address. */
    
#endif /* ICL_SLAVE_ENABLE */

#define RIDGE_CMD_CCG_RESET                        (0x02)
/**< Alpine/Titan Ridge command value that requests a CCG device reset. */

#define RIDGE_CMD_INT_CLEAR                        (0x04)
/**< Alpine/Titan Ridge command value to clear the interrupt from CCG. */

#define RIDGE_IRQ_ACK                              (0x2000)
/**< Alpine/Titan Ridge command IRQ ACK PD Controller to Titan Ridge. */

#if BB_RETIMER_ENABLE
    
#define RIDGE_CMD_WRITE_TO_RETIMER                 (0x10)
/**< Ice Lake command value to request to write to the retimer */    
    
#define RIDGE_STS_RETIMER_DATA_VALID               (0x400000u)
/**< Ice Lake status bit to indicate that retimer data is valid */
    
#endif /* BB_RETIMER_ENABLE */    

/**
 * @brief List of Alpine/Titan Ridge slave interface registers.
 *
 * The Thunderbolt Alternate Mode specification defines the following set of registers
 * that should be implemented by a USB-PD port controller in Thunderbolt enabled systems.
 */
typedef enum
{
    RIDGE_REG_CONTROLLER_STATE = 0x03,          /**< PD controller state register. */
    RIDGE_REG_CCG_COMMAND      = 0x50,          /**< Data Control register. */
    RIDGE_REG_CCG_STATUS       = 0x5F,          /**< Data Status register. */
    RIDGE_REG_RETIMER_DEBUG    = 0x5D           /**< Retimer debug register */
} ridge_slave_reg_addr_t;

/**
 * @typedef irq_state_t
 * @brief List of states of IRQ queue/dequeue state machine.
 */
typedef enum
{
    DFP_IRQ_DEFAULT_STATE,
    DFP_IRQ_SENT_TO_TR,
    DFP_IRQ_CLR_BY_CCG,
    DFP_IRQ_CLR_ACK_BY_TR,
}irq_state_t;

/**
 * @brief Initialize the Alpine/Titan Ridge slave interface module.
 *
 * This function initializes the Alpine/Titan Ridge slave interface module and configures it
 * to use the specified SCB block. The SCB will be configured as an I2C slave block,
 * and the interrupt output will also be initialized to a de-asserted state.
 *
 * Since only two registers are to be implemented, and the commands to be implemented are
 * simple, the complete module is implemented using the I2C command callbacks.
 *
 * @param scbnum SCB index to be used for the Alpine/Titan Ridge slave interface.
 * @param portsel Alpine/Titan Ridge slave port selection. Only applicable for single port designs.
 *
 * @return None
 */
void ridge_slave_init(uint8_t scbnum, uint8_t portsel);

/**
 * @brief De-initialize the Alpine/Titan Ridge slave interface module.
 *
 * This function de-initializes the SCB block used to implement the I2C slave interface
 * between CCGx and Alpine/Titan Ridge.
 */
void ridge_slave_deinit(void);

/**
 * @brief Update the AR/TR status register and send an event to the Alpine/Titan Ridge.
 *
 * This function is used by the application layer to update the content of the Alpine/Titan
 * Ridge status register. If the content of the register is changing, CCG asserts
 * the corresponding interrupt to notify Alpine/Titan Ridge about the status change.
 *
 * @param port USB-PD port index corresponding to the status update.
 * @param status Value to be written into the status register.
 * @param rewrite Flag to enable updating of the status register even it remains the same.
 *
 * @return None
 */
void ridge_slave_status_update(uint8_t port, uint32_t status, bool rewrite);

/**
 * @brief Handler for pending Ridge Slave interface tasks.
 * @return None
 */
void ridge_slave_task(void);

/**
 * @brief Check whether the AR/TR slave interface is idle so that device can be placed into sleep.
 *
 * This function should be called prior to placing the CCG device in deep sleep. Deep sleep
 * entry is only allowed if this function returns true.
 *
 * @return true if the interface is idle, false otherwise.
 */
bool ridge_slave_sleep(void);

/**
 * @brief Resets Ridge related registers.
 * @param port USB-PD port index corresponding to the register reset.
 * @return None
 */
void ridge_reg_reset(uint8_t port);

/**
 * @brief Checks if any PMC/Ridge update is pending
 * @return uint8_t Bitmask of updates pending 
 */
uint8_t ridge_update_is_pending(void);

/**
 * @brief Check whether a host is connected to the specified PD port.
 * @param port Index of the PD port.
 * @return true if a host is connected, false otherwise.
 */
bool ridge_slave_is_host_connected(uint8_t port);

/**
 * @brief Update the data status register to indicate OverCurrent fault condition.
 * @param port PD port index.
 * @return None
 */
void ridge_slave_set_ocp_status(uint8_t port);

/**
 * @brief Enable or disable HPI notifications about SOC Ack timeout condition. This API is only valid
 * on Ice Lake / Tiger Lake platforms and will be a NOP otherwise.
 *
 * @param disable Whether HPI notification on SOC Ack timeout should be disabled.
 * @return None
 */
void ridge_slave_soc_timeout_event_control(bool disable);

#if BB_RETIMER_ENABLE
/**
 * @brief Update the NIDnT overlay number
 * @param port USB-PD port index corresponding to the status update.
 * @param overlay_num Overlay number
 * @return None
 */
void ridge_slave_set_ovnum(uint8_t port, uint8_t overlay_num);

/**
 * @brief Update the AR/TR status register with debug info and send an event to the Alpine/Titan Ridge.
 * @param port USB-PD port index corresponding to the status update.
 * @param debug Debug Mode data
 * @return None
 */
void ridge_slave_set_debug(uint8_t port, uint32_t debug);

/**
 * @brief Clear the Write_To_Retimer bit in the ridge slave cmd register.
 * @param port USB-PD port index corresponding to the status update.
 * @return None
 */
void ridge_slave_clear_write_to_retimer_bit(uint8_t port);

/**
 * @brief Enable/disable delayed soc alter notification to the TBT controller.
 * @param enable Whether notifications should be delayed.
 * @return None
 */
void ridge_delayed_soc_alert(bool enable);

#endif /* BB_RETIMER_ENABLE */


#endif /* _RIDGE_SLAVE_H_ */

/* [] END OF FILE */
