/**
 * @file tcc.c
 *
 * @brief @{Test code coverage source file.@}
 */

/*
 * Copyright (2014-2020), Cypress Semiconductor Corporation or a subsidiary of
 * Cypress Semiconductor Corporation. All rights reserved.
 *
 * This software, including source code, documentation and related materials
 * (“Software”), is owned by Cypress Semiconductor Corporation or one of its
 * subsidiaries (“Cypress”) and is protected by and subject to worldwide patent
 * protection (United States and foreign), United States copyright laws and
 * international treaty provisions. Therefore, you may use this Software only
 * as provided in the license agreement accompanying the software package from
 * which you obtained this Software (“EULA”).
 *
 * If no EULA applies, Cypress hereby grants you a personal, nonexclusive,
 * non-transferable license to copy, modify, and compile the Software source
 * code solely for use in connection with Cypress’s integrated circuit
 * products. Any reproduction, modification, translation, compilation, or
 * representation of this Software except as specified above is prohibited
 * without the express written permission of Cypress. Disclaimer: THIS SOFTWARE
 * IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress reserves the
 * right to make changes to the Software without notice. Cypress does not
 * assume any liability arising out of the application or use of the Software
 * or any product or circuit described in the Software. Cypress does not
 * authorize its products for use in any products where a malfunction or
 * failure of the Cypress product may reasonably be expected to result in
 * significant property damage, injury or death (“High Risk Product”). By
 * including Cypress’s product in a High Risk Product, the manufacturer of such
 * system or application assumes all risk of such use and in doing so agrees to
 * indemnify Cypress against all liability.
 */

#include <stdint.h>
#include <tcc.h>
#include <tcc_id.h>
#include <config.h>

#if TEST_CODE_COVERAGE

/* Size of raw data returned for each raw data get call. */
#define TCC_RAW_DATA_EXTRACT_BLOCK      (4)

/*
 * Byte array to store raw data for test code coverage.
 * Each bit in the array corresponds to execution status of function in func_id_t.
 * If function is executed, then corresponding bit is set to '1'.
 */
static uint8_t gl_tcc_raw_data[(FUNC_ID_COUNT > 8) ? ((FUNC_ID_COUNT >> 3) + 1) : 1];

void tcc_init(void)
{
    uint8_t state = 0;

    state = CyEnterCriticalSection();
    memset(gl_tcc_raw_data, 0x00, sizeof(gl_tcc_raw_data));
    CyExitCriticalSection(state);
}

void tcc_set_func_bit(func_id_t func_id)
{
    uint8_t bit_pos = 0;
    uint8_t state = 0;
    uint32_t array_index = 0;

    state = CyEnterCriticalSection();
    array_index = func_id >> 3;
    bit_pos = func_id & 0x7;

    gl_tcc_raw_data[array_index] |= (1u << bit_pos);
    CyExitCriticalSection(state);
}

uint8_t tcc_get_percent(void)
{
    uint8_t i = 0;
    uint8_t j = 0;
    uint8_t state = 0;
    uint32_t percent = 0;

    state = CyEnterCriticalSection();
    for (i = 0; i < sizeof(gl_tcc_raw_data); i++)
    {
        for(j = 0; j < 8; j++)
        {
            if(gl_tcc_raw_data[i] & (1u << j))
            {
                percent++;
            }
        }
    }

    /* Calculate percentage w.r.t total of active functions */
    percent = (uint8_t)((percent * 100)/FUNC_ACTIVE_COUNT);
    CyExitCriticalSection(state);

    return percent;
}

uint32_t tcc_get_raw_data_size(void)
{
    return sizeof(gl_tcc_raw_data);
}

uint32_t tcc_get_raw_data(uint8_t * data, uint32_t offset, uint32_t size)
{
    uint8_t state;
    int32_t tcc_data_left = (tcc_get_raw_data_size() - offset + 1);

    state = CyEnterCriticalSection();
    if ((data != NULL) || (tcc_data_left <= 0))
    {
        uint32_t bytes;
		if (tcc_data_left >= size)
        {
            bytes = size;
        }
        else
        {
            bytes = tcc_data_left;
            memset(data, 0x00, size);
        }
        memcpy(data, (gl_tcc_raw_data + offset), bytes);
    }
    CyExitCriticalSection(state);

    return bytes;
}

#endif  /* TEST_CODE_COVERAGE  */

/* End of file */
